/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Widget Library
*
* (C) Copyright IBM Corp. 2013.  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
/******************************************************* {COPYRIGHT-TOP-RM} ***
* Licensed Materials - Property of IBM
*
* "Restricted Materials of IBM"
*
* 5724-C51
* (C) Copyright IBM Corp. 2010 All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
******************************************************* {COPYRIGHT-END-RM} ***/

/*
 * Base dialog class used by both the ErrorDialog and ViewDialog widgets.
 * This class provides common methods and shared resources to limit
 * duplications. 
 */
define("dojoe/timeselector/BaseDialog", [
        "dojoe/timeselector/Util",
        "dijit/_WidgetBase",
        "dijit/_TemplatedMixin",
        "dijit/_WidgetsInTemplateMixin",
        "dijit/Dialog",
        "dojo/html",
        "dojo/parser",
        "dojo/_base/lang",
        "dojo/_base/array",
        "dojo/_base/connect",
        "dijit/registry",
        "dojo/_base/declare"
        ], function(Util, _WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, Dialog, html, parser, lang, array, connect, registry, declare){
	return declare("dojoe.timeselector.BaseDialog", [_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin],{
		// Used for Templated widget
		   //widgetsInTemplate: true,
		   templateString: null,
		   
		   //iContext passed in by iWidget
		   iContext: null,
		   
		   //Underlying based dialog id
		   dialogId: null,
		   
		   //Arrays of all dojo.connect()
		   connections: [],
		   
		   /*
		    * Constructs a BaseDialog dijit
		    */
		   constructor: function (/*Object*/ kwArgs) {
			   lang.mixin(this,kwArgs);
		   },
		   
		   /*
		    * Invoked after the DOM fragment is added to the document.
		    * Used for customized initializations
		    */
		   startup: function(){
		      this.inherited(arguments);
		      this._startup();
		   },
		   
		   /*
		    * Invoked after the DOM fragment is created.
		    * Used to set connect onHide call back method
		    */
		   postCreate: function() {
			  this.inherited(arguments);
			  this.addConnect("onHide","onHide");
		   },

		   /* 
		    * Invoked to destroy this widget.
		    * Used to disconnect all previosuly dojo.connect()
		    */
		   destroy: function() {
			  this.inherited(arguments);
			  array.forEach(this.connections,connect.disconnect);
			  this.connections = [];
		   },

		   /* 
		    * Retrieves the Dojo dijit for specified id
		    */
		   getWidget: function(/*String*/ id) {
		      return registry.byId(id);
		   },
		   
		   /*
		    * Sets the value for specified widget attribute name and 
		    * and id
		    */
		   setWidgetValue: function(/*String*/ id, /*String*/ attrName, /*Object*/ attrValue) {
		      var widget = this.getWidget(id);
		      if(widget != null) {
		         widget.set(attrName,attrValue);
		      }
		   },
		   
		   /*
		    * Retrieves the value for specified attribute name
		    * and widget id
		    */
		   getWidgetValue: function(/*String*/ id, /*String*/ attrName) {
		      var widget = this.getWidget(id);
		      if(widget != null) {
		         return widget.get(attrName);
		      }
		      return null;
		   },
		   
		   /*
		    * Creates and adds a dojo.connect for the specified event name
		    * and call back method
		    */
		   addConnect: function(/*String*/ eventName, /*String*/ methodName){
		      var baseDialog = this._getBaseDialog();
		      if(baseDialog != null) {
		          this.connections.push(connect.connect(baseDialog,eventName,this,methodName));
		      }
		   },
		   
		   /*
		    * Sets the underlying base dialog id
		    */
		   setDialogId: function(/*String*/ dialogId) {
		      this.dialogId = dialogId;
		   },
		   
		   /*
		    * Determines if widget should be persisted. May be over-ridden for 
		    * customization during persistence determination.
		    * By default, return true.
		    */
		   isPersist: function(/* Dijit Widget*/ widget){
		      return true;
		   },
		   
		   /*
		    * Retrieves the persisted value for the widget. May be over-ridden for 
		    * customization during persistence determination.
		    * By default, return the "value" for the value attribute. Null, if widget is
		    * undefined. 
		    */
		   getPersist: function(/*Dijit Widget*/ widget) {
		      if(widget == null) {
		         return null;
		      }
		      return widget.get("value");
		   },
		   
		   /*
		    * Retrieves the ManagedItemSet object for this dialog.
		    */
		   getManagedItemSet: function(){
			  if ( this.iContext ) {
				  return this.iContext.getiWidgetAttributes();
			  } else {
				  return null;
			  }
		   },
		   
		   /* 
		    * Persists the dialog by storing name and value pairs.
		    * Also persists specified hash array of name value pairs, if possible.
		    */
		   persist: function(/*Array*/ hashArray){
		      if(this.iContext != null) {
		           //Switch to personalize mode to allow persisting per user time configuration preference
		           this.iContext.iEvents.fireEvent("onModeChanged",null,"{newMode:'personalize'}");		  
		           var managedItemSet = this.getManagedItemSet();
		           if(managedItemSet != null) {
		               //Process all specified name-value pairs
		               for(var i in hashArray){
		                  managedItemSet.setItemValue(i,hashArray[i]); 
		               }
		               this._persist(hashArray,managedItemSet);//Customized persist may be necessary
		      	       managedItemSet.save();
		      	   }
		      	   //Switch back to view 
				   this.iContext.iEvents.fireEvent("onModeChanged",null,"{newMode:'view'}");	         
		      }
		   },
			
		   /* 
		    * Launches the dialog 
		    */        
		   show: function(){
		      var baseDialog = this._getBaseDialog();
		      if(baseDialog != null) {
		         baseDialog.show();
		      }
		   },
		   
		   /* 
		    * Hides the dialog 
		    */
		   hide: function(){
		      var baseDialog = this._getBaseDialog();
		      if(baseDialog != null) {
		         baseDialog.hide();
		      }
		   }, 	   
		       
		   /* 
		    * Call back when dialog is hidden 
		    */
		   onHide: function(){}, 
		   
		   /* 
		    * Retrieves the underlying base dialog 
		    */
		   _getBaseDialog: function(){
			  //var newDialog = new Dialog({},this.dialogId);
			  //return newDialog;
		      return this.getWidget(this.dialogId);
		   },
		   
		   /* 
		    * Called during widget startup().
		    * May be over-ridden for further customized dialog initializations.
		    */
		   _startup: function(){},
		   
		   /*
		    * Called during dialog persist. 
		    * May be over-ridden for further customized dialog persistence.
		    */
		   _persist: function(/*Array*/ hashArray, /*ManagedItemSet*/ managedItemSet) {}	
	});
});
