/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Widget Library
*
* (C) Copyright IBM Corp. 2013.  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
/******************************************************* {COPYRIGHT-TOP-RM} ***
* Licensed Materials - Property of IBM
*
* "Restricted Materials of IBM"
*
* 5724-C51
* (C) Copyright IBM Corp. 2010 All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
******************************************************* {COPYRIGHT-END-RM} ***/

define("dojoe/timeselector/Util", [
        "dojo/_base/declare",
        "dojo/date/locale",
        "dojo/date/stamp",
        "dojo/i18n!dojoe/timeselector/nls/Common",
        "dojo/i18n!dojoe/timeselector/nls/Messages",
		"dojo/_base/lang"
        ], function(declare, date, stamp, Common, Messages, lang){
	
	return declare("dojoe.timeslector.Util", null, {
		//Logging: Boolean
		   logging: true,
		   timezoneDescArr: null,
		   timezoneOffSetArr: null,
		   
		   /*
		    * Constructs a Util with specified boolean or object for logging 
			* output to the the browser console and/or set timezonDescArr. 
			* 	logging
			* 	timezoneDescArr - an array of time zone description.
			* 	timezoneOffSetArr - an array of time zone offset.
		    */
		   constructor: function (/*Object or boolean*/ args) {
			   	if(lang.isObject(args)) lang.mixin(this, args);
				else this.logging = args;
			   	if(!this.timezoneDescArr) {
					if(typeof timezoneDescArr === "undefined")
				   		this.timezoneDescArr = [];
					else 
						this.timezoneDescArr = timezoneDescArr;
				}
			   	if(!this.timezoneOffSetArr) {
					if(typeof timezoneOffSetArr === "undefined")
				   		this.timezoneOffSetArr = [];
					else 
						this.timezoneOffSetArr = timezoneOffSetArr;
				}
			   //this.i18nHash = dojo.i18n.getLocalization("dojoe.timeselector", "Common");
			   //this.i18nMsgs = dojo.i18n.getLocalization("dojoe.timeselector", "Messages");
		   },
		   
		   /*
		    * Sets the logging.
		    */
		   setLogging: function(/*Boolean*/ logging) {
		       this.logging = logging;
		   },
		 	
		   /*
		    * Converts specified valid string to a date object and return it. Null if string invalid.
		    */
		   toDate: function(/*String*/ dateString) {
		       try {
		          return new Date(Date.parse(dateString));
		       }
		       catch(e){
		          this.log(e);
		       }
		       return null;
		   },
		   
		   getTimeZoneOffSetValue: function(tzDesc) {
				var retVal = "";
				for(i=0;i<this.timezoneDescArr.length;i++) {
					if(this.timezoneDescArr[i] == tzDesc) {
						retVal = this.timezoneOffSetArr[i];
					}
				}
				console.info("getTimeZoneOffSetValue:"+tzDesc+" offset:"+retVal);
				return retVal;		
			},

		   /*
		    * Converts the specified date based on specified time zone.
		    */
		   toLocaleDateAndTime: function(/*Date*/ theDate, /*String*/ timeZone){
		      var offSet = 0;
			  
			  offSet = (typeof getTimeZoneOffSetValue !== "undefined")?getTimeZoneOffSetValue(timeZone):this.getTimeZoneOffSetValue(timeZone);
			  offSet = offSet.replace(".30", ".50");
			  offSet = offSet.replace(".45", ".75");
			  offSet = offSet.replace(".-", ".");
			 
			  var localTime = theDate.getTime();
			  var localOffset = theDate.getTimezoneOffset()*60000;
			  var localUTC = localTime+localOffset;
			  var targetTime = localUTC + (3600000*offSet);		
		      return new Date(targetTime);  
		   },
		   
		   /*
		    * Creates and return a unique Id based on current time.
		    */
		   getUniqueId: function(){
		      return new Date().getTime();
		   },
		    
		   /*
		    * Retrieves the localized string for specified key.
		    */
		   getLocalizedString: function(/*String*/ key) {
			  //this.constructor(true);
		      //return i18nHash.key;
		   },
		   
		   /*
		    * Retrieves the localized message for specified key.
		    */
		   getLocalizedMessage: function(/*String*/ key) {
		      //return i18nMsgs.key;
		   },
		   
		   /*
		    * Converts specified date to an ISO based string.
		    */ 
		   toISOString: function(/*Date*/ theDate) {
		       return dojo.date.stamp.toISOString(theDate,{selector:"date"}).replace(/-/g,"")+dojo.date.stamp.toISOString(theDate,{selector:"time"}).replace(/:/g,"")+"Z";
		   },	   
		    
		   /*
		    * Writes the specified object to the browser console, if available.
		    */     
		   log: function(/*Object*/ obj){
		       if(window.console && this.logging && (obj != null)){ 
		          console.log(obj);
		       }
		   },	
		    
		   /*
		    * Formats the specified date as specified. Returns a full year date, if as is not recognized.
		    */ 
		   formatDateAs: function(/*Date*/ theDate, /*String*/ as) {
		       if(as=="date"){
			      return dojo.date.locale.format(theDate,{selector:"date",fullYear: true});
			   }
			   else if(as=="time"){
			      return dojo.date.locale.format(theDate,{selector:"time"});
			   }
			   else if(as=="zone"){
			      //return dojo.date.locale.format(theDate,{formatLength:"short",selector:"time",timePattern:"zz"});
				  return '';
			   }
			   return dojo.date.locale.format(theDate,{fullYear: true});
		   },
		   
		   formatDateFromPackAs: function(/*Date*/ theDate, /*Object*/ pack, /*String*/ as) {
		       if(as=="date"){
			      return pack.format(theDate,{selector:"date",fullYear: true});
			   }
			   else if(as=="time"){
			      return pack.format(theDate,{selector:"time"});
			   }
			   else if(as=="zone"){
			      return pack.format(theDate,{formatLength:"short",selector:"time",timePattern:"zz"});
			   }
			   return pack.format(theDate,{fullYear: true});
		   }
		   
	});
});
