/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Widget Library
*
* (C) Copyright IBM Corp. 2013.  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
define("dojoe/treetablex/_Toolbar", [
	"dojo/_base/declare",
	"dojoe/common/toolbar/_Toolbar",
	"dijit/form/Button",
	"dijit/ToolbarSeparator",
	"dijit/Menu",
	"dijit/MenuItem",
	"dijit/form/DropDownButton",
	"dojo/i18n!dojoe/treetable/nls/TreeTable",
	"dojoe/treetablex/_ConfigureColumnsDialog",
	"dojoe/treetablex/_QuickSearchFilter"
	], function (declare, _CommonToolbar, Button, ToolbarSeparator, Menu, MenuItem, DropDownButton, _resources, _ConfigureColumnsDialog, _QuickSearchFilter) {


	_Toolbar = declare('dojoe.treetablex._Toolbar', [_CommonToolbar], {	
	
		// summary: 
		//		A private class to create tree table toolbar with icons, action lists and fitler options
		// description: 
		//		This is based on a common toolbar widget which supports both table and tree table widgets.
		//		This class is internally used to enable/disable toolbar icons, actions and search controls in the tree table
		//		Please refer dojoe.treetable.TreeTable.toolbarOptions property.
		// tags: private
		
		// widgetsInTemplate: boolean
		//		to indicate widgets in template
		widgetsInTemplate:true,
		//toolbar:null,
		
		// treeTable: object
		//		Reference to tree table instance
		treeTable:null,
		
		// actionListWidget: object
		//		Reference to Action List menu in toolbar
		actionListWidget:null,
		
				
		// refreshIcon: boolean
		//		To enable/disable refresh icon on toolbar, defaults to true
		refreshIcon:true,
		
		// refreshMenu: boolean
		//		To enable/disable refresh menu item in the action list 
		refreshMenu:true,
		
		// showSelectAllIcon: boolean
		//		to enable/disable select All icon in the toolbar
		showSelectAllIcon:false, //55028 - Default for TreeTable Widget
		
		// onlySelectAllIcon: boolean
		//		To provide a option to use only select All without de-select All option
		onlySelectAllIcon:false,
		
		// onlyDeselectAllIcon: boolean
		//		To provide a option to use only de-select All without select All option	
		onlyDeselectAllIcon:false,
		
		//09Oct2012 [54541]: Latha : Adding the attributes to provide an option to use expandAll & collapseAll individually and also retaining the property expandAllIcon for compatibility
		// expandAllIcon: boolean
		//		to indicate the state of items expand/collapse when the tree table loads
		expandAllIcon: false, //55028 - Default for TreeTable Widget
		
		// onlyExpandAllIcon: boolean
		//		To provide a option to use only expand All without collapse All option
		onlyExpandAllIcon:false,
		
		// onlyCollapseAllIcon: boolean
		//		To provide a option to use only collapse All without expand All option	
		onlyCollapseAllIcon:false,
		
		
		// clearSortIcon: boolean
		//		To enable/disable clear sort icon on the toolbar, defaults to true
		clearSortIcon: false,  //55028 - Default for TreeTable Widget
		
		// configureTreeTableIcon: boolean
		//		To enable/disable Configure options in the toolbar, defaults to true
		configureTreeTableIcon:false, //55028 - Default for TreeTable Widget
		
		// filterOptions: object
		//		filter options to set to quick filter
		filterOptions: null,
		
		// actionMenu: boolean
		//		To enable/disable Action list in the toolbar, defaults to true
		actionMenu:true,
		
		// selectAllMenu: boolean
		//		To enable/disable select All menu item in the Action list, defaults to true
		selectAllMenu: true,
		
		// expandAllMenu: boolean
		//		To enable/disable expandAll menu item in Action list, defaults to true
		expandAllMenu : true,
		
		// clearSortMenu: boolean
		//		To enable/disable clear sort menu item in Action list, defaults to true
		clearSortMenu: true,
		
		// configureTreeTableMenu: boolean
		//		To enable/disable configure options menu item in Action list, defaults to true
		configureTreeTableMenu:true,
		
		// toolbarOptions: object
		//		local variable which holds the tree table toolbarOptions values 
		toolbarOptions : null,
		
		// quickFilterVisible: boolean
		//		To enable/disable quick search filter on the toolbar, defaults to true
		quickFilterVisible : true,
		
		// quickFilterVisible: boolean
		//		To enable/disable quick filter button on the toolbar, defaults to false
		showQuickFilterButton: false,	
		
		// _gridOrigStructure : object
		// Persisting the grid original structure 
		_gridOrigStructure : null,
		
		// Constructor function. Called when instance of this class is created
		constructor : function() {
			// summary:
			//		constructor
		
		},
		
		postCreate:function(){
			// summary:
			//		To handle stuff when the toolbar is rendered
			// tags: extension
			
			this.filterOptions = this.treeTable.filterOptions;
			this.toolbarOptions = this.treeTable.toolbarOptions;
			
			//Latha : 54640: Unable to open filter popup for TreeTable and Tree Table gridx, As advanced filter is not yet enabled on TreeTablex hiding the filterbar by default
			if(this.filterOptions.hideFilterBar == undefined) {
				this.filterOptions.hideFilterBar = true;
			}
			if(this.filterOptions && this.filterOptions.showQuickFilter !== undefined){
				this.quickFilterVisible = this.filterOptions.showQuickFilter;
			}	
			if(this.filterOptions && this.filterOptions.showQuickFilterButton !== undefined){
				this.showQuickFilterButton = this.filterOptions.showQuickFilterButton;
			}
			if(this.filterOptions.hideFilterBar == true && this.treeTable._grid.filterBar !== undefined){
				this.treeTable._grid.filterBar.hide();
			}
			if(this._gridOrigStructure == null) {
				//Ramachandran - 24-Apr-13 - 72840: TreeTable: Config options dialog not showing the hidden columns in widget def.
				//_gridOrigStructure was refering to modified structure instead of original structure.
				this._gridOrigStructure = this.treeTable._origLayout;
			}
			this._mapToolbarControls();
			this._createToolBarControls();			
		},
		
		_mapToolbarControls : function() {
			// summary: 
			//		Internal function to map toolbar options from dojoe.treetable.TreeTable to properties of this class
	
			if(this.toolbarOptions) {
				if(this.toolbarOptions.refreshIcon == false){
					this.refreshIcon = this.toolbarOptions.refreshIcon;
				}
				if(this.toolbarOptions.refreshMenu == false){
					this.refreshMenu = this.toolbarOptions.refreshMenu;
				}				
				if(this.toolbarOptions.selectAllIcon == false){
					this.showSelectAllIcon = this.toolbarOptions.selectAllIcon;
					
				}else{
					if(this.toolbarOptions.onlySelectAllIcon == true){
						this.onlySelectAllIcon=this.toolbarOptions.onlySelectAllIcon;
					}
					if(this.toolbarOptions.onlyDeselectAllIcon == true){
						this.onlyDeselectAllIcon=this.toolbarOptions.onlyDeselectAllIcon;
					}
				}
				if(this.toolbarOptions.selectAllMenu == false){
					this.selectAllMenu = this.toolbarOptions.selectAllMenu;
				}				
				if(this.toolbarOptions.expandAllIcon == false){
					this.expandAllIcon = this.toolbarOptions.expandAllIcon;
				}else{
					if(this.toolbarOptions.onlyExpandAllIcon == true){
						this.onlyExpandAllIcon=this.toolbarOptions.onlyExpandAllIcon;
					}
					if(this.toolbarOptions.onlyCollapseAllIcon == true){
						this.onlyCollapseAllIcon=this.toolbarOptions.onlyCollapseAllIcon;
					}
				}
				if(this.toolbarOptions.expandAllMenu == false){
					this.expandAllMenu = this.toolbarOptions.expandAllMenu;
				}
				
				if(this.toolbarOptions.clearSortIcon == false){
					this.clearSortIcon = this.toolbarOptions.clearSortIcon;
				}
				if(this.toolbarOptions.clearSortMenu == false){
					this.clearSortMenu = this.toolbarOptions.clearSortMenu;
				}
				
				if(this.toolbarOptions.configureTreeTableIcon == true) //55028 - Default for TreeTable Widget
				{
					this.configureTreeTableIcon = this.toolbarOptions.configureTreeTableIcon;
				}
				if(this.toolbarOptions.configureTreeTableMenu == false){
					this.configureTreeTableMenu = this.toolbarOptions.configureTreeTableMenu;
				}
				if(this.toolbarOptions.actionMenu == false){
					this.actionMenu = this.toolbarOptions.actionMenu;
				}
				if(this.toolbarOptions.quickFilterVisible == false){
					this.quickFilterVisible = this.toolbarOptions.quickFilterVisible;
				}	
				if(this.toolbarOptions.quickFilterVisible && this.toolbarOptions.showQuickFilterButton){
					this.showQuickFilterButton = this.toolbarOptions.showQuickFilterButton;
				}
								
			}			
		},
		
		_refreshTreeTable: function(){
			// summary:
			//		Internal function to invoke tree table refresh
			
			this.treeTable.refresh();
		},
		
		_selectAll: function () 
		{
			// summary:
			//		Internal function to select all the rows in the tree table, provided Indirect Selection is enabled
			var count = this.treeTable._grid.rowCount();
				for (var i=0; i<count; i++) {
					this.treeTable._grid.row(i).select();
				}
			
		},
		
		_deselectAll: function () 
		{
			// summary:
			//		Internal function to deselect all the rows in the tree table, provided Indirect selection is enabled
			
			this.treeTable._grid.select.row.clear();
					
		},
		
		_expandAll : function(){
		
			// summary:
			//		Internal function to expand all the parent items in the tree
		
			var count = this.treeTable._grid.rowCount();
				for (var i=0; i<count; i++) {
					this.treeTable._grid.tree.expandRecursive(this.treeTable._grid.row(i).id);
				}
		},
		
		_collapseAll : function()
		{
			// summary:
			//		Internal function to collapse all the parent items in the tree
			
			var count = this.treeTable._grid.rowCount();
				for (var i=0; i<count; i++) {
					this.treeTable._grid.tree.collapseRecursive(this.treeTable._grid.row(i).id);
				}
		},
		
		_clearSort : function()
		{
			// summary:
			//		Internal function to clear sorting applied to all the columns
			
			this.treeTable._grid.sort.clear();
		},
		
		_configureColumn : function()
		{ 
			// summary:
			//		Internal function to show configure options dialog, which provides ability to show/hide columns in the tree table
			
		    this.tableOptionsDialog = new _ConfigureColumnsDialog ({ treeTable:this.treeTable,indirectSelection:this.treeTable.indirectSelection,enableColumnReorder:this.treeTable.enableColumnReorder, gridStructure: this._gridOrigStructure});
			this.tableOptionsDialog.show();
		},
		
		//16716: table toolbar has browser popup menu on right click
		_stopEvent: function(e){
			// summary:
			//		to stop event propagation
			
			//Since event is oncontextmenu, just stop it!
			dojo.stopEvent(e);				
		},
		
		_createToolBarControls: function(){
			// summary:
			//		Internal function specific to tree table used to create required toolbar options/controls
			
			if(this.refreshIcon) {
				var refreshTableIcon = new Button({
					id: this.id+"refreshIcon",
					showLabel: false,
					label: _resources.REFRESH,
					title: _resources.REFRESH,
					iconClass: "refreshIcon"
				});
					
				refreshTableIcon.onClick = dojo.hitch (this, "_refreshTreeTable");
				refreshTableIcon.onKeypress = dojo.hitch (this, "_refreshTreeTable");
				this._addStateControl(refreshTableIcon,0);
				
				// 53655 : Support for mobile mode
				if(!this.treeTable.mobileMode) {
					var seperator = new ToolbarSeparator({id: "refSeperator"+this.id});			
					this._addStateControl(seperator);
				}
			}
			if(this.showSelectAllIcon) {	
				if(this.onlySelectAllIcon){
					var selectAllIcon = new Button({
						id: this.id+"selectAllIcon",
						showLabel: false,
						label: _resources.SELECT_ALL,
						title: _resources.SELECT_ALL,
						iconClass: "selectAllIcon"
					});
						
					selectAllIcon.onClick = dojo.hitch (this, "_selectAll");
					selectAllIcon.onKeypress = dojo.hitch (this, "_selectAll");
					
					this._addFastPathControl(selectAllIcon);
				} else if(this.onlyDeselectAllIcon){
					var deselectAllIcon = new Button({
						id: this.id+"deselectAllIcon",
						showLabel: false,
						label: _resources.DESELECT_ALL,
						title: _resources.DESELECT_ALL,
						iconClass: "deselectAllIcon"
					
					});
						
					deselectAllIcon.onClick = dojo.hitch (this, "_deselectAll");
					deselectAllIcon.onKeypress = dojo.hitch (this, "_deselectAll");
					
					this._addFastPathControl(deselectAllIcon);
				} else {
					var selectAllIcon = new Button({
						id: this.id+"selectAllIcon",
						showLabel: false,
						label: _resources.SELECT_ALL,
						title: _resources.SELECT_ALL,
						iconClass: "selectAllIcon"
						
					});
						
					selectAllIcon.onClick = dojo.hitch (this, "_selectAll");
					selectAllIcon.onKeypress = dojo.hitch (this, "_selectAll");
					
					this._addFastPathControl(selectAllIcon);
					
					var deselectAllIcon = new Button({
						id: this.id+"deselectAllIcon",
						showLabel: false,
						label: _resources.DESELECT_ALL,
						title: _resources.DESELECT_ALL,
						iconClass: "deselectAllIcon"
						
					});
						
					deselectAllIcon.onClick = dojo.hitch (this, "_deselectAll");
					deselectAllIcon.onKeypress = dojo.hitch (this, "_deselectAll");
					this._addFastPathControl(deselectAllIcon);
				}
			}
			
			
			if(this.expandAllIcon) {
				if(this.onlyExpandAllIcon) {
				var expandtAllIcon = new dijit.form.Button({
						id: this.id+"expandableAllIcon",
						showLabel: false,
						label: _resources.EXPAND_ALL,
						title: _resources.EXPAND_ALL,
						iconClass: "expandtAllIcon"
					},this.expandtAllIcon);
					
				expandtAllIcon.onClick = dojo.hitch (this, "_expandAll");
				expandtAllIcon.onKeypress = dojo.hitch (this, "_expandAll");
				this._addFastPathControl(expandtAllIcon);
			} else if(this.onlyCollapseAllIcon) {
				var collapseAllIcon = new Button({
					id: this.id+"collapseAllIcon",
					showLabel: false,
					label: _resources.COLLAPSE_ALL,
					title: _resources.COLLAPSE_ALL,
					iconClass: "collapseAllIcon"
					
				});				
				collapseAllIcon.onClick = dojo.hitch (this, "_collapseAll");
				collapseAllIcon.onKeypress = dojo.hitch (this, "_collapseAll");			
				this._addFastPathControl(collapseAllIcon);
			} else {
				var expandtAllIcon = new dijit.form.Button({
						id: this.id+"expandableAllIcon",
						showLabel: false,
						label: _resources.EXPAND_ALL,
						title: _resources.EXPAND_ALL,
						iconClass: "expandtAllIcon"
					},this.expandtAllIcon);
					
				expandtAllIcon.onClick = dojo.hitch (this, "_expandAll");
				expandtAllIcon.onKeypress = dojo.hitch (this, "_expandAll");
				// 54125  Do NOT hardcode disabling expand all as it can be turned off via
				//        toolbarOptions.expandAllIcon = false
				// Temporarily disable expand all until we get performance
				// issue for gridx and our data store fixed
				// expandtAllIcon.set("disabled",true);
				this._addFastPathControl(expandtAllIcon);
				
				var collapseAllIcon = new Button({
					id: this.id+"collapseAllIcon",
					showLabel: false,
					label: _resources.COLLAPSE_ALL,
					title: _resources.COLLAPSE_ALL,
					iconClass: "collapseAllIcon"
					
				});				
				collapseAllIcon.onClick = dojo.hitch (this, "_collapseAll");
				collapseAllIcon.onKeypress = dojo.hitch (this, "_collapseAll");			
				this._addFastPathControl(collapseAllIcon);
				}
			} 
			if(this.clearSortIcon) {
				var removeSortIcon = new Button({
					id: this.id+"clearSortIcon",
					showLabel: false,
					label: _resources.CLEAR_SORT,
					title: _resources.CLEAR_SORT,
					iconClass: "removeSortIcon"
					
				});
					
				removeSortIcon.onClick = dojo.hitch (this, "_clearSort");
				removeSortIcon.onKeypress = dojo.hitch (this, "_clearSort");
				this._addFastPathControl(removeSortIcon);
			}
			
			if(this.configureTreeTableIcon) {
				var configureColIcon = new dijit.form.Button({
					id: this.id+"configureTreeTableIcon",
					showLabel: false,
					title: _resources.CONFIGURE_TREETABLE,
					label: _resources.CONFIGURE_TREETABLE,
					iconClass: "configureColIcon"
				});
					
				configureColIcon.onClick = dojo.hitch (this, "_configureColumn");
				configureColIcon.onKeypress = dojo.hitch (this, "_configureColumn");
				this._addFastPathControl(configureColIcon);
			} 
			
			if(this.quickFilterVisible && this.showQuickFilterButton) {
				this.quickFilter = new  _QuickSearchFilter({
					id: this.id+"quickFilter", 
					grid: this.treeTable._grid,
					_quickFilterButton: true,
					filterOptions: this.filterOptions },this.quickFilterNode);	
				var quickFilterIcon = new dijit.form.Button({
					showLabel: false,
					iconClass: "filterIcon",
					id: "quickFilterIcon"+this.id,
					label: _resources.QUICK_FILTER,
					title: _resources.QUICK_FILTER
				},this.filterButton);				
				quickFilterIcon.onClick = dojo.hitch (this, "_setQuickFilter");
				quickFilterIcon.onKeypress = dojo.hitch (this, "_setQuickFilter");
				quickFilterIcon.domNode.title = _resources.QUICK_FILTER;	
			} else if(this.quickFilterVisible && !this.showQuickFilterButton) {
				this.quickFilter = new _QuickSearchFilter({
					id: this.id+"quickFilter", 
					grid: this.treeTable._grid,
					_quickFilterButton: false,
					filterOptions: this.filterOptions },this.quickFilterNode);				
			}
			
			if(this.actionMenu){
				var menu = new Menu({ style: "display: none;"});
						
				if(this.refreshMenu) {
					var refreshMenuItem = new MenuItem({
						title: _resources.REFRESH,
						label: _resources.REFRESH,
						iconClass: "refreshIcon"
					});
					refreshMenuItem.onClick = dojo.hitch (this, "_refreshTreeTable");
					refreshMenuItem.onKeypress = dojo.hitch (this, "_refreshTreeTable");					 
					menu.addChild(refreshMenuItem);
				}		

				//55028 - Default for TreeTable Widget
				if(this.configureTreeTableMenu) {				
					var configureColMenuItem = new MenuItem({
						title: _resources.CONFIGURE_TREETABLE,
						label: _resources.CONFIGURE_TREETABLE,
						iconClass: "configureColIcon"
					});
					configureColMenuItem.onClick = dojo.hitch (this, "_configureColumn");
					configureColMenuItem.onKeypress = dojo.hitch (this, "_configureColumn");					 
					menu.addChild(configureColMenuItem);
				}		
				
				if(this.selectAllMenu){
					var selectAllMenuItem = new MenuItem({
						 title: _resources.SELECT_ALL,
						 label: _resources.SELECT_ALL,
						 iconClass: "selectAllIcon"			         
					 });
					 selectAllMenuItem.onClick = dojo.hitch (this, "_selectAll");
					 selectAllMenuItem.onKeypress = dojo.hitch (this, "_selectAll");	
					 menu.addChild(selectAllMenuItem);
					
					 var deselectAllMenuItem = new MenuItem({
						  title: _resources.DESELECT_ALL,
						 label: _resources.DESELECT_ALL,
						 iconClass: "deselectAllIcon"
					  });
					 deselectAllMenuItem.onClick = dojo.hitch (this, "_deselectAll");
					 deselectAllMenuItem.onKeypress = dojo.hitch (this, "_deselectAll");
					 menu.addChild(deselectAllMenuItem);
				}
				
				if(this.expandAllMenu){
					
					var expandAllMenuItem = new dijit.MenuItem({
						 title: _resources.EXPAND_ALL,
						 label: _resources.EXPAND_ALL,
						 iconClass: "expandtAllIcon"
					 });
					 expandAllMenuItem.onClick = dojo.hitch (this, "_expandAll");
					 expandAllMenuItem.onKeypress = dojo.hitch (this, "_expandAll");
					 // 54125  Do NOT hardcode disabling expand all as it can be turned off via
					 //        toolbarOptions.expandAllMenu = false
					 // Temporarily disable expand all until we get performance
					 // issue for gridx and our data store fixed
					 // expandAllMenuItem.set("disabled",true);
					 menu.addChild(expandAllMenuItem);
					 
					 var collapseAllMenuItem = new MenuItem({
						 title: _resources.COLLAPSE_ALL,
						 label: _resources.COLLAPSE_ALL,
						 iconClass: "collapseAllIcon"
					 });
					 collapseAllMenuItem.onClick = dojo.hitch (this, "_collapseAll");
					 collapseAllMenuItem.onKeypress = dojo.hitch (this, "_collapseAll");
					 menu.addChild(collapseAllMenuItem);
				}
					
				if(this.clearSortMenu){
					 var clearSortMenuItem = new MenuItem({
						label: _resources.CLEAR_SORT,
						title: _resources.CLEAR_SORT,
						iconClass: "removeSortIcon"
					 });
					 clearSortMenuItem.onClick = dojo.hitch (this, "_clearSort");
					 clearSortMenuItem.onKeypress = dojo.hitch (this, "_clearSort");
					 menu.addChild(clearSortMenuItem);
				}
				
				
				menu.startup();
				var actionList = new DropDownButton({
					optionsTitle : _resources.ACTIONS,
			        label: _resources.ACTIONS,
			        dropDown: menu,
					scrollOnFocus : true
			    });				
				this.actionListWidget = actionList;		
				this._addFastPathControl(actionList);	
			}
		},
		
		_setQuickFilter: function(){
			// summary:
			//		Internal function to set filter criteria for quick filter
			
			this.quickFilter._setFilter();
		},	
		
		_removeFromActionList: function (menuItemId){
			// summary:
			//		Internal function invoked by dojoe.treetable.TreeTable to remove items form Action list on the toolbar
			
			var menuItem = dijit.byId(menuItemId);
			if(menuItem !== null && menuItem !== undefined) {
				menuItem.destroyRecursive();
			}
		},
		
		//do not call this method directly, use table.removeItemFromToolbar(itemId) method to remove a custom item from the toolbar
		_removeItem: function(itemId){
			// summary:
			//		Internal function invoked by dojoe.treetable.TreeTable to remove controls on the toolbar
			
			var button= dijit.byId(itemId);
			if(button !== null && button !== undefined) {		
				if (button.domNode.parentNode) {
					//remove td which is the parent of the button
					buttonParent = button.domNode.parentNode;
					buttonParent.parentNode.removeChild(buttonParent);
				}
				button.destroyRecursive();
			}
		},
		
		//do not call this method directly, use treetable.addToToolbar(item,position,row) method to add a custom item to the toolbar before default buttons
		_addCustomControl: function(item, position, row){
			// summary:
			//		Internal function invoked by dojoe.treetable.TreeTable to add custom buttons, actions to the toolbar
			
			this.inherited(arguments);
		},	

		destroy: function(){
			// summary:  Destroys the toolbar.
			// tags: public	
			
			//memory leak issue: custom destroy method to take care of removing all widgets					
			try{
				delete _resources;
				//this is for destroying child templated widgets 
				//this should happen by itself but need to figure out why dijit._Templated is not taking care of it...
				var widgets = dijit.findWidgets(this.domNode);
				dojo.forEach(widgets, function(w) {
					if(w){
						if(w.destroyRecursive){
							w.destroyRecursive();
						} else if(w.destroy){
							w.destroy();
						}
					}
				});
				// this is for widgets added as child to this widget and their children..
				//not present in template
				var widgetArr = this.getChildren();
				dojo.forEach(widgetArr, function (child) {
					if(child){
						if(child.destroyRecursive){
							child.destroyRecursive();
						} else if(child.destroy){
							child.destroy();
						}
					}
				});
				this.inherited(arguments);
			}catch(ex){
				console.warn("Widget destroy failed: "+this.id+", ex="+ex.name+":"+ex.message);
			}
		}
	});
	return _Toolbar;
});
