/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Widget Library
*
* (C) Copyright IBM Corp. 2013.  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
define("dojoe/treetablex/modules/StatefulRefresh", [
                                                    "dojo",
                                                    "dijit",
                                                    'dojo/_base/array', 
                                                    "gridx/modules/Tree",
                                                    "gridx/core/_Module",
                                                    "dojo/_base/Deferred",
                    								"dojo/DeferredList"
                                                    ], 
function(dojo, dijit, Array, Tree, _Module, Deferred, DeferredList) {
	
    _Module.register(Tree);
    
	return  dojo.declare('dojoe.treetablex.modules.StatefulRefresh', _Module, {
		
		// [Module Dependency Management] --------------------------------------------
		
		//Every module needs a name. 
		//It could be the same as another module's, which means that module's public API is the same as this one's.
		//If this name is omitted, the long class name will be used ('dojoe.treetablex.modules.EnhancedRefresh' in this case),
		name: 'statefulRefresh',

		constructor: function(){
			this._clear();
		},
			
		//Required dependency modules must also exist if this module exists.
		//But they don't need to finish loading before this one.
		required: ['tree'],
		
		_expStatus: null,
		
		
		// [Module API Management] ---------------------------------------------------
		
		getAPIPath: function(){
			//This method defines how to access this module's API from the grid widget.
			//Anything returned from this function is 'deeply' mixed into grid.
			//For this example, we'll be able to call:
			//		grid.enhancedRefresh
			
			//If some other module needs to extend the APIs in namespace grid.testMode,
			//it can also provide this method in exact the same way, because the mixing in
			//process is performed layer by layer (deeply).
			
			//You can even directly add methods to grid: 
			//		return {
			//			newMethodForGrid: function(){...}
			// 		};
			
			return {
				statefulRefresh: this
			};
		},
		
	
	
		// [Module Lifetime Management] -----------------------------------------------
		preload: function(args){
			//This is called after all modules are created, but not loaded yet. Dependancy sequence is not hornored here.
			var t = this,
			g = t.grid;
			t.batchConnect(
				[g.tree,'onExpand','_recordExpand'],
				[g.tree,'onCollapse','_recordCollapse'],
				[g, 'setStore', '_clear']
				//[g.body, 'refresh', '_persist']
				);
		},
	
		load: function(args, deferStartup){
			var t = this;
			
		},
		
		_applyTreeState: function(){
			var t = this;
			/*53045: LS - 9/27/2012 - clearing the cache again after refresh causes the store lose the cache...*/
			//t.grid.model.clearCache();
			//t.grid.tree._clear();
			if(t.grid.persist){
				var content = t.grid.persist.get(t.grid.id);
				if(content && content["expStatus"]){
					var expandoStatusArr = content["expStatus"];
					var len = expandoStatusArr[''].opened.length;
					var i, deferredList = [];
					for(i=0; i<len; i++){
						var deferred = null;
						if(t._canExpand(expandoStatusArr[''].opened[i]) && !t.grid.tree.isExpanded(expandoStatusArr[''].opened[i])){
							if(expandoStatusArr[''].opened.length > 0) {
								deferred = t.grid.tree.expand(expandoStatusArr[''].opened[i],true);
								deferredList.push(deferred );
							}
						}
					}
					if(deferredList.length > 0) {
					new DeferredList(deferredList).then(function(){
								t.grid.body.updateRootRange(t.grid.body.rootStart,t.grid.body.rootCount);
								t.grid.body.refresh();
							});
					}				
				}
			}

		},
		
		/*53045: LS - 9/27/2012 - TreeTableView stops working after refreshing when previous nodes were in expanded state...*/
		/*check for haschildren before expanding the node in case the nodes and the structure has changed...*/
		_canExpand: function(id){
			// summary:
			//		Check whether a row can be expanded.
			// id: String
			//		The row ID
			// returns:
			//		Whether the row can be expanded.
			var t = this,
			m = t.grid.model;
			return  m.hasChildren(id, m.store._getItem(id));
		},
		
		_recordExpand: function(id){
			if(!this._expStatus[id]){
				this._expStatus[id] = { id: id,
										status:'open'
									   };
				this._expStatus[''].opened.push(id);						
				this._persist();	
			}			
			
		},
		_recordCollapse: function(id){
			delete(this._expStatus[id]); 
			var arr = this._expStatus[''].opened;
			Array.forEach(arr, function(openId,i){
				if(openId === id){
					// Latha : 58772: IE expects two arguments for arr.splice index and count
					arr.splice(i,1);
				}
			});
			this._persist();
		},
		
		_persist: function(){
			// persist the visible column information
			if(this.grid.persist){
				var expStatus = this._expStatus;
				var status = {'expStatus': expStatus};
				this.grid.persist.key = this.grid.id;
				this.grid.persist.put(this.grid.persist.key, status);
				this.grid.persist.registerAndLoad('expStatus', function(){
							return expStatus;
						});
				this.grid.persist.save();
			}
		},
		_clear: function(){
			this._expStatus = {'': {id: '', status: '', opened:[]} };
			
		},
		
		destroy: function(){
			this.inherited(arguments);
			this._clear();
			//do something to tear me down here.
			//e.g.: dojo.destroy(this.domNode);
		}
	
	
	});
});