/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define(
        "ibm/usmi/statuspod/AlertStatusPod", [
            "ibm/tivoli/tip/TCTTable",
            "ibm/tivoli/tip/TCTNavModel",
            "dojo/_base/kernel",
            "dojo/_base/declare",
            "dojo/_base/lang",
            "dojo/_base/connect",
            "dijit/PopupMenuBarItem",
            "./StatusPodMenuDialog",
            "./AlertStatusPodDialogContent",
            "ibm/usmi/util/DataStore",
            "dojo/i18n!./nls/statuspodResources",
            "ibm/usmi/core/utils/_UsmiBrowserPersistence",
            "dojo/DeferredList",
            "dojo/_base/Deferred",
            'dojo/_base/unload',
            "ibm/usmi/util/DefaultMessageHandler",
            "./Constants",
            "ibm/usmi/core/utils/Logger"
        ],
        function(TCTTable, TCTNavModel, dojo, declare, lang, connect, PopupMenuBarItem, StatusPodMenuDialog,
                AlertStatusPodDialogContent, DataStore, MRI, _UsmiBrowserPersistence, DeferredList, Deferred,
                baseUnload, DefaultMessageHandler, Constants, Logger) {
            return declare(
                    "ibm.usmi.statuspod.AlertStatusPod",
                    [
                        PopupMenuBarItem,
                        _UsmiBrowserPersistence
                    ],
                    {

                        // summary:
                        //      Alert Status Pod widget 
                        // description:
                        //      The Status summary widget would show the systems with active alerts including Problems and Non-Compliance issues.
                        //      It would show the Systems with Problems or Non-Compliances issues in the same look and feel as mega menu.

                        label: MRI.StatusPodLoading,

                        dialog: null,
                        systemCounterDS: null,
                        problemsDataset: null,
                        complianceDataset: null,
                        problemHandler: null,
                        complianceHandler: null,
                        menuParent: null,
                        blockProblemsEvents: true,
                        pendingProblemsEvents: false,
                        blockCompliancesEvents: true,
                        pendingCompliancesEvents: false,
                        totalCounter: 0,

                        _PERSISTENCE_ALERTS_PROBLEMS_ID: Constants.PERSISTENCE_ALERTS_PROBLEMS_ID,
                        _PERSISTENCE_ALERTS_NONCOMPLIANCE_ID: Constants.PERSISTENCE_ALERTS_NONCOMPLIANCE_ID,
                        _PERSISTENCE_ALERTS_TOTALCOUNT: Constants.PERSISTENCE_ALERTS_TOTALCOUNT,

                        problemsGroupProperties: [
                            "DisplayName",
                            "ObjectType",
                            "STATUS",
                            "STATUS_CAT:Problems",
                            "STATUSPOD_SORT:Problems"
                        ],

                        compliancesGroupProperties: [
                            "DisplayName",
                            "ObjectType",
                            "STATUS",
                            "STATUS_CAT:ComplianceStatusCategory",
                            "STATUSPOD_SORT:ComplianceStatusCategory"
                        ],

                        problemsGroupSort: [ {
                            attribute: "STATUSPOD_SORT:Problems",
                            descending: true
                        }
                        ],

                        problemsGroupSortGetItems: "-STATUSPOD_SORT:Problems",

                        nonCompliancesGroupSort: [ {
                            attribute: "STATUSPOD_SORT:ComplianceStatusCategory",
                            descending: true
                        }
                        ],

                        nonCompliancesGroupSortGetItems: "-STATUSPOD_SORT:ComplianceStatusCategory",

                        _logger: new Logger("ibm.usmi.statuspod", "AlertStatusPod"),

                        problemsCache: null,

                        nonCompliancesCache: null,

                        postCreate: function() {
                            this.inherited(arguments);
                            this.dialog = new AlertStatusPodDialogContent();
                            this.popup = new StatusPodMenuDialog({
                                content: this.dialog
                            });
                        },

                        startup: function() {
                            this.inherited(arguments);
                            this.createDatasets();
                            connect.subscribe(Constants.TOPIC_CLOSE_ALERTS_DIALOG, this, function(result) {
                                this.popup.close();
                            });
                        },

                        destroy: function() {
                            // clean up the datastore
                            this.destoryDataSets();
                            if (this.popup) {
                                this.popup.close();
                            }
                            if (this.dialog) {
                                this.dialog.destroyRecursive();
                            }
                            this.dialog = null;
                            this.inherited(arguments);
                        },

                        destoryDataSets: function() {
                            // summary:
                            //      Destroy the datasets for this widget
                            if (this.systemCounterDS) {
                                this.systemCounterDS.getDataset().removeEventListener(this);
                                this.systemCounterDS.destroy();
                                this.systemCounterDS = null;
                            }
                            if (this.problemsDataset) {
                                this.problemsDataset.getDataset().removeEventListener(this);
                                this.problemsDataset.destroy();
                                this.problemsDataset = null;
                            }
                            if (this.complianceDataset) {
                                this.complianceDataset.getDataset().removeEventListener(this);
                                this.complianceDataset.destroy();
                                this.complianceDataset = null;
                            }
                        },

                        createDatasets: function() {
                            // summary:
                            //      Create the datasets for systems with Problems, system with Non-Compliance, and the systems with either
                            //      Problems or Non-Compliance alerts
                            var problemsCache = this.getData(this._PERSISTENCE_ALERTS_PROBLEMS_ID, true);
                            var nonCompliancesCache = this.getData(this._PERSISTENCE_ALERTS_NONCOMPLIANCE_ID, true);
                            if (problemsCache && nonCompliancesCache) {
                                this._refreshProblems(problemsCache, true);
                                this._refreshCompliance(nonCompliancesCache, true);
                                this._updateTopElement(true);
                                this.problemsCache = problemsCache;
                                this.nonCompliancesCache = nonCompliancesCache;
                            }

                            this.systemCounterHandler = new Deferred();
                            this._createStatusSystemsCounterDataset();

                            this.problemHandler = new Deferred();
                            this.complianceHandler = new Deferred();

                            var handlers = new DeferredList([
                                this.problemHandler,
                                this.complianceHandler,
                                this.systemCounterHandler
                            ]);

                            handlers.then(lang.hitch(this, function(result) {
                                this._updateTopElement();
                                window
                                        .setTimeout(dojo.hitch(this, this._unblockProblemsEvents),
                                                Constants.RERESH_DELAY);
                                window.setTimeout(dojo.hitch(this, this._unblockCompliancesEvents),
                                        Constants.RERESH_DELAY);
                            }));

                            this.problemsDataset = this.makeDataSet("SYSTEM_STATUSPOD", lang.hitch(this,
                                    this.initSetupProblemsMenu), this.problemsGroupProperties, {
                                groupId: "STATUS_PROBLEMS_SUMMARY",
                                STATUS_CAT: "Problems"
                            }, this.problemsGroupSort);

                            this.complianceDataset = this.makeDataSet("SYSTEM_STATUSPOD", lang.hitch(this,
                                    this.initSetupCompliancesMenu), this.compliancesGroupProperties, {
                                groupId: "STATUS_COMPLIANCE_SUMMARY",
                                STATUS_CAT: "ComplianceStatusCategory"
                            }, this.nonCompliancesGroupSort);

                        },

                        initSetupProblemsMenu: function(items) {
                            // summary:
                            //      create problems menu after got the items data
                            // items: ibm.tivoli.navmodel.Item[]
                            //      the top list of systems with Problems
                            var ds = this.problemsDataset;
                            if (ds) {
                                ds.getDataset().addEventListener(this, lang.hitch(this, this.dynamicRefreshProblems));
                            }
                            //if the cache items equals the first fetched items, then do not refresh it
                            if (this._checkIfCachedItemsChanged(this.problemsCache, items, true)) {
                                this._logger.finest("items is different than problems cache, need to refresh",
                                        "initSetupProblemsMenu");
                                if (this.problemsCache) {
                                    this.dialog.clearProblemsMenu();
                                }
                                this._refreshProblems(items);
                            }
                            if (this.problemHandler) {
                                this.problemHandler.resolve();
                            }

                        },

                        initSetupCompliancesMenu: function(items) {
                            // summary:
                            //      create compliances menu after got the items data
                            // items: ibm.tivoli.navmodel.Item[]
                            //      the top list of systems with Non-Compliances
                            var ds = this.complianceDataset;
                            if (ds) {
                                ds.getDataset()
                                        .addEventListener(this, lang.hitch(this, this.dynamicRefreshCompliances));
                            }
                            if (this._checkIfCachedItemsChanged(this.nonCompliancesCache, items, false)) {
                                this._logger.finest("items is different than nonCompliances cache, need to refresh",
                                        "initSetupCompliancesMenu");
                                if (this.nonCompliancesCache) {
                                    this.dialog.clearCompliancesMenu();
                                }
                                this._refreshCompliance(items);
                            }
                            if (this.complianceHandler) {
                                this.complianceHandler.resolve();
                            }
                        },

                        initSetupSystemsCounter: function() {
                            // summary:
                            //      setup the listener to refresh the total counter for the systems with Problems or Non-Compliances
                            var ds = this.systemCounterDS;
                            if (ds) {
                                ds.getDataset().addEventListener(this, lang.hitch(this, this.dynamicRefreshTotalCount));
                            }
                            this._updateTotalCount();
                            if (this.systemCounterHandler) {
                                this.systemCounterHandler.resolve();
                            }
                        },

                        _checkIfCachedItemsChanged: function(cachedItems, currentItems, isProblem) {
                            // summary:
                            //      compare the new fetched items with the cached items
                            // cachedItems: ibm.tivoli.navmodel.Item[]
                            //      the cached item list
                            // currentItems: ibm.tivoli.navmodel.Item[]
                            //      the new fetched item list
                            // isProblem: boolean
                            //      the flag to indicate if it's a Problems list or not
                            var i;
                            if (!cachedItems && !currentItems) {
                                //no change
                                return false;
                            }

                            if (!cachedItems && currentItems) {
                                //changed
                                return true;
                            }

                            if (cachedItems && !currentItems) {
                                //changed
                                return true;
                            }

                            cachedItems = cachedItems || [];
                            currentItems = currentItems || [];

                            if (cachedItems.length !== currentItems.length) {
                                //items length change
                                return true;
                            }

                            for (i = 0; i < cachedItems.length; i++) {
                                var cachedItem = cachedItems[i];
                                var curItem = this.convertItemToCachedItem(currentItems[i], isProblem);
                                if (!this.isItemEqual(cachedItem, curItem)) {
                                    //item is not equal
                                    return true;
                                }
                            }

                            return false;

                        },

                        convertItemToCachedItem: function(item, isProblem) {
                            // summary:
                            //      convert the ibm.tivoli.navmodel.Item to cached item object
                            // item: ibm.tivoli.navmodel.Item
                            //      the ibm.tivoli.navmodel.Item retrieved by the rest call
                            // isProblem: boolean
                            //      the flag to indicate if it's a Problem item or not
                            var cacheItem, statuscatName, sortPropName, severityId, severityValue, sourceName, objectType, itemId;
                            var sortPropVal;
                            var totalItems = item.getDataset().getItemCount();
                            if (isProblem) {
                                statuscatName = "STATUS_CAT:Problems";
                                sortPropName = "STATUSPOD_SORT:Problems";
                            } else {
                                statuscatName = "STATUS_CAT:ComplianceStatusCategory";
                                sortPropName = "STATUSPOD_SORT:ComplianceStatusCategory";
                            }
                            severityId = item.getProperty(statuscatName).getValue().getId();
                            severityValue = item.getProperty(statuscatName).getValue().getValue();
                            sourceName = item.getProperty("DisplayName").getValue();
                            objectType = item.getProperty("ObjectType").getDisplayValue();
                            itemId = item.getId();
                            sortPropVal = item.getProperty(sortPropName).getValue();
                            cacheItem = {
                                severityId: severityId,
                                sourceName: sourceName,
                                objectType: objectType,
                                severityValue: severityValue,
                                itemId: itemId,
                                sortPropVal: sortPropVal,
                                totalCounter: totalItems
                            };
                            return cacheItem;
                        },

                        isItemEqual: function(cacheItem, currentCacheItem) {
                            // summary:
                            //     compare two cached item objects
                            // cacheItem: Object
                            //      the cached item object
                            // currentCacheItem: Object
                            //      the other cached item object
                            var i;
                            if (cacheItem.severityId !== currentCacheItem.severityId) {
                                return false;
                            }
                            if (cacheItem.sourceName !== currentCacheItem.sourceName) {
                                return false;
                            }
                            if (cacheItem.objectType !== currentCacheItem.objectType) {
                                return false;
                            }
                            if (cacheItem.severityValue !== currentCacheItem.severityValue) {
                                return false;
                            }
                            if (cacheItem.itemId !== currentCacheItem.itemId) {
                                return false;
                            }
                            if (cacheItem.totalCounter !== currentCacheItem.totalCounter) {
                                return false;
                            }
                            var oldSortPropVal = cacheItem.sortPropVal;
                            var curSortPropVal = currentCacheItem.sortPropVal;
                            if (oldSortPropVal && curSortPropVal) {
                                if (oldSortPropVal.length !== curSortPropVal.length) {
                                    return false;
                                }
                                for (i = 0; i < oldSortPropVal.length; i++) {
                                    if (oldSortPropVal[i] !== curSortPropVal[i]) {
                                        return false;
                                    }
                                }
                            }
                            return true;
                        },

                        _needToProcessEvent: function(event, interestingProperties) {
                            // summary:
                            //     check to see if we need to handle the event
                            // description:
                            //      Check the event received, if the event contains only the properties would not effect the status pod, we would ignore
                            //      such event
                            // event: ibm.tivoli.navmodel.Event
                            //      the event received
                            // interestingProperties: String[]
                            //      the property id list that we care about, if those property is changed, we need to refresh the menu
                            var eventType, changedProps, eventProps, eventProp, interestingProp, itemIdx, eventIdx, interestingIdx;
                            if (!event) {
                                //if there is no event passed in like the case to process the pending refresh, then we must to refresh it
                                return true;
                            }
                            eventType = event.getType();
                            if (eventType !== "itemPropertiesChanged") {
                                //not properties change event, we need to handle it
                                return true;
                            }
                            changedProps = event.getProperties();
                            if (!changedProps || changedProps.length === 0) {
                                //no properties in properties changed event
                                return false;
                            }

                            if (!interestingProperties) {
                                //no interestingProperties passed, then assume we need to process it
                                return true;
                            }

                            for (itemIdx = 0; itemIdx < changedProps.length; itemIdx++) {
                                eventProps = changedProps[itemIdx];
                                if (!eventProps || eventProps.length === 0) {
                                    continue;
                                }
                                for (eventIdx = 0; eventIdx < eventProps.length; eventIdx++) {
                                    eventProp = eventProps[eventIdx];
                                    for (interestingIdx = 0; interestingIdx < interestingProperties.length; interestingIdx++) {
                                        interestingProp = interestingProperties[interestingIdx];
                                        if (interestingProp === eventProp.getId()) {
                                            return true;
                                        }
                                    }
                                }
                            }
                            if (this._logger.isFinest()) {
                                this._logger.finest("not need to process this properties changed event",
                                        "_needToProcessEvent");
                            }
                            return false;
                        },

                        dynamicRefreshProblems: function(listenerObject, event) {
                            // summary:
                            //     refresh the problems menu based on the event
                            // listenerObj: Object
                            //      unique object
                            // event: ibm.tivoli.navmodel.Event
                            //      the event received

                            //for property change event, we only care about the properties id we are interesting with
                            if (!this._needToProcessEvent(event, this.problemsGroupProperties)) {
                                return;
                            }

                            if (this.blockProblemsEvents) { //Refresh is in progress, don't refresh
                                this.pendingProblemsEvents = true; //save the need to refresh again later
                                return;
                            }
                            this.blockProblemsEvents = true;
                            this.pendingProblemsEvents = false;
                            this.dialog.clearProblemsMenu();
                            this.label = MRI.StatusPodRefreshing;
                            this.containerNode.innerHTML = this.label;
                            dojo.byId('alertStatusPodHeaderText').innerHTML = MRI.StatusHeaderTextRefreshing;

                            if (this.problemHandler) {
                                delete this.problemHandler;
                            }
                            this.problemHandler = new Deferred();
                            this.problemHandler.then(lang.hitch(this, function(result) {
                                this._updateTopElement();
                                window
                                        .setTimeout(dojo.hitch(this, this._unblockProblemsEvents),
                                                Constants.RERESH_DELAY);
                            }));

                            var dataset = this.problemsDataset.getDataset();
                            dataset.getItems({
                                onComplete: lang.hitch(this, function(items, kwArgs) {
                                    this._refreshProblems(items);
                                    if (this.problemHandler) {
                                        this.problemHandler.resolve();
                                    }
                                }),
                                onError: function(error, exception, ioArgs) {
                                    var msgHandler = new DefaultMessageHandler({});
                                    msgHandler.displayException(exception);
                                },
                                sort: this.problemsGroupSortGetItems,
                                properties: this.problemsGroupProperties,
                                scope: this,
                                count: 10,
                                parameters: {
                                    groupId: "STATUS_PROBLEMS_SUMMARY",
                                    STATUS_CAT: "Problems"
                                }
                            });
                            this._logger.finest("re-fecthing systems in problems group", "dynamicRefreshProblems");
                        },

                        dynamicRefreshCompliances: function(listenerObject, event) {
                            // summary:
                            //     refresh the non-compliances menu based on the event
                            // listenerObj: Object
                            //      unique object
                            // event: ibm.tivoli.navmodel.Event
                            //      the event received

                            if (!this._needToProcessEvent(event, this.compliancesGroupProperties)) {
                                this._logger.finest("ignore the event", "dynamicRefreshCompliances");
                                return;
                            }

                            if (this.blockCompliancesEvents) { //Refresh is in progress, don't refresh
                                this.pendingCompliancesEvents = true; //save the need to refresh again later
                                return;
                            }
                            this.blockCompliancesEvents = true;
                            this.pendingCompliancesEvents = false;
                            this.dialog.clearCompliancesMenu();
                            this.label = MRI.StatusPodRefreshing;
                            this.containerNode.innerHTML = this.label;
                            dojo.byId('alertStatusPodHeaderText').innerHTML = MRI.StatusHeaderTextRefreshing;

                            if (this.complianceHandler) {
                                delete this.complianceHandler;
                            }
                            this.complianceHandler = new Deferred();
                            this.complianceHandler.then(lang.hitch(this, function(result) {
                                this._updateTopElement();
                                window.setTimeout(dojo.hitch(this, this._unblockCompliancesEvents),
                                        Constants.RERESH_DELAY);
                            }));

                            var dataset = this.complianceDataset.getDataset();
                            dataset.getItems({
                                onComplete: lang.hitch(this, function(items, kwArgs) {
                                    this._refreshCompliance(items);
                                    if (this.complianceHandler) {
                                        this.complianceHandler.resolve();
                                    }
                                }),
                                onError: function(error, exception, ioArgs) {
                                    var msgHandler = new DefaultMessageHandler({});
                                    msgHandler.displayException(exception);
                                },
                                sort: this.nonCompliancesGroupSortGetItems,
                                properties: this.compliancesGroupProperties,
                                scope: this,
                                count: 10,
                                parameters: {
                                    groupId: "STATUS_COMPLIANCE_SUMMARY",
                                    STATUS_CAT: "ComplianceStatusCategory"
                                }
                            });
                            this._logger.finest("re-fecthing systems in noncompliances group",
                                    "dynamicRefreshCompliances");
                        },

                        dynamicRefreshTotalCount: function() {
                            // summary:
                            //     refresh the total counter for the alert status pod
                            this._updateTotalCount();
                            this._updateTopElement();
                        },

                        _unblockProblemsEvents: function() {
                            // summary:
                            //     unblock the event handling for the problems event

                            this.blockProblemsEvents = false; //Refresh is done, we can now listen for events again
                            if (this.pendingProblemsEvents) { //did any more events come in during our refresh? if so, refresh again
                                this.dynamicRefreshProblems();
                            }
                        },

                        _unblockCompliancesEvents: function() {
                            // summary:
                            //     unblock the event handling for the non-compliances event

                            this.blockCompliancesEvents = false; //Refresh is done, we can now listen for events again
                            if (this.pendingCompliancesEvents) { //did any more events come in during our refresh? if so, refresh again
                                this.dynamicRefreshCompliances();
                            }
                        },

                        _refreshProblems: function(items, fromCache) {
                            // summary:
                            //     refresh the Problems menu

                            this.dialog.updateProblems(items, fromCache);
                            //refresh problems persistent cache
                            this.setData(this._PERSISTENCE_ALERTS_PROBLEMS_ID, this.dialog.cache.problems, true);
                        },

                        _refreshCompliance: function(items, fromCache) {
                            // summary:
                            //     refresh the Non-Compliance menu

                            this.dialog.updateCompliances(items, fromCache);
                            //refresh nonCompliances persistent cache
                            this.setData(this._PERSISTENCE_ALERTS_NONCOMPLIANCE_ID, this.dialog.cache.compliance, true);

                        },

                        makeDataSet: function(datasetId, callback, groupProperties, parameters, sortProp) {
                            // summary:
                            //     create and return the DataStore utility 
                            // datasetId: String
                            //     the id of the dataset
                            // callback: Function
                            //     callback function to handle the items of the dataset
                            // groupProperties: String[]
                            //    properties id list that need to fetched
                            // parameters: Object
                            //    parameters for the backend collection
                            // sortProp: Object
                            //    sorting properties
                            return new DataStore({
                                datasetId: datasetId,
                                getTasks: false,
                                initReadyCallback: callback
                            }, {
                                sort: sortProp,
                                scope: this,
                                count: 10,
                                defaultProperties: groupProperties,
                                clearCacheOnRefresh: true,
                                parameters: parameters
                            });
                        },

                        _createStatusSystemsCounterDataset: function() {
                            // summary:
                            //     create the dataset for the total counter of the systems with Problems or Non-Compliances
                            if (this.systemCounterDS) {
                                this.systemCounterDS.destroy();
                                this.systemCounterDS = null;
                            }
                            this.systemCounterDS = new DataStore({
                                datasetId: 'GROUP:SYSTEMS_STATUS_SUMMARY',
                                getItems: false,
                                retrieveItemProperties: false,
                                getTasks: false,
                                initReadyCallback: lang.hitch(this, this.initSetupSystemsCounter)
                            });
                        },

                        _updateTotalCount: function() {
                            // summary:
                            //    update the total counter value
                            var ds;
                            if (!this.systemCounterDS) {
                                return;
                            }
                            ds = this.systemCounterDS.getDataset();
                            this.totalCounter = ds.getItemCount();
                            this.setData(this._PERSISTENCE_ALERTS_TOTALCOUNT, this.totalCounter, false);
                            this._logger.finest("update total count of the alert systems", "_updateTotalCount");
                        },

                        _updateTopElement: function(fromCache) {
                            // summary:
                            //    update the total counter value and image in the menu bar item, and also the explanation text in the dialog
                            // fromCache: Boolean
                            //    flag to indicate if it's based on cached data or not
                            var problemItems = this.dialog.statusItems, complianceItems = this.dialog.complianceItems;
                            if (fromCache) {
                                this.totalCounter = this.getData(this._PERSISTENCE_ALERTS_TOTALCOUNT, false);
                            }
                            var tc = this.totalCounter;
                            dojo.byId('alertStatusPodHeaderText').innerHTML = lang.replace(MRI.StatusHeaderText, [ tc
                            ]);
                            var overallStatusIcon = this.determineOverallStatusIcon(this
                                    .determineStatusIcon(problemItems), this.determineStatusIcon(complianceItems));
                            this.label = "<span class='" + overallStatusIcon + "'></span>"
                                    + lang.replace(MRI.Status, [ tc
                                    ]);
                            this.containerNode.innerHTML = this.label;
                        },

                        determineStatusIcon: function(items) {
                            // summary:
                            //    determine each category overall status
                            // description:
                            //    determine the problems category or the non-compliance category status
                            // items: ibm.tivoli.navmodel.Item[]
                            //    the items to determine the category status
                            var overallStatusIcon = Constants.STATUS_CLASS_NORMAL;
                            if (!items || items.length === 0) {
                                return overallStatusIcon;
                            }
                            var item = items[0];
                            var statusVal = item.severityId;
                            if (statusVal === 1) {
                                overallStatusIcon = Constants.STATUS_CLASS_WARNING;
                            } else if (statusVal === 0) {
                                overallStatusIcon = Constants.STATUS_CLASS_ERROR;
                            } else if (statusVal === 5) {
                                overallStatusIcon = Constants.STATUS_CLASS_UNKNOWN;
                            }
                            return overallStatusIcon;
                        },

                        determineOverallStatusIcon: function(statusProblems, statusCompliances) {
                            // summary:
                            //    determine the overall status for the alert status pod
                            // statusProblems: String
                            //    overall status for the systems with problems
                            // statusCompliances: String
                            //    overall status for the systems with non-compliances
                            if (statusProblems === Constants.STATUS_CLASS_ERROR
                                    || statusCompliances === Constants.STATUS_CLASS_ERROR) {
                                return Constants.STATUS_CLASS_ERROR;
                            }
                            if (statusProblems === Constants.STATUS_CLASS_WARNING
                                    || statusCompliances === Constants.STATUS_CLASS_WARNING) {
                                return Constants.STATUS_CLASS_WARNING;
                            }

                            if (statusProblems === Constants.STATUS_CLASS_NORMAL
                                    && statusCompliances === Constants.STATUS_CLASS_UNKNOWN) {
                                return Constants.STATUS_CLASS_UNKNOWN;
                            }
                            return Constants.STATUS_CLASS_NORMAL;
                        }
                    });
        });
