/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/util/DefaultTableDAHelper", [
	"dojo/_base/kernel",
	"dojo/_base/declare",
    "dojo/topic",
	// NextGen imports
	"ibm/tivoli/tip/navwidget/topology/dijit/DefaultActionHelper",
	"ibm/tivoli/navmodel/Selection",
	"ibm/usmi/core/utils/Logger"
], function (dojo, declare, topic, DefaultActionHelper, Selection, Logger) {

	return declare([ DefaultActionHelper
	], {

		runCustomAction : true,
		_args : null,
		_task : null,
		urlProperty : "URL",
		
		// the logger
		_daLogger : new Logger("ibm.usmi.widgets.util", "DefaultTableDAHelper"),

		/*
		 * Method which check context for default server task and executes if needed
		 * else executes the callback method
		 */
		executeDefaultAction : function (/*Object*/ objectContext, /*Property*/ property) {

			this._args = {
				launchParms : "",
				menuItem : "",
				context : ""
			};

			dojo.mixin(this._args, objectContext);

			if(property){				
				this.executePropertyAction(objectContext, property);				
			} else {
				/*
				var selectedObjects = objectContext.selection, object, oid, URL, currentLocation;
				if ((selectedObjects.getLinkCount() + selectedObjects.getItemCount()) !== 1) {
					//if there is more than one item then just return and do nothing for now
					return;
				}
	
				object = null;
	
				//set the selected object
				if (selectedObjects.getItemCount() === 1) {
					object = selectedObjects.getItems()[0];
				}
	
				//var URL = (object.getProperty(this.urlProperty)).getValue();
				//TODO 2928
				oid = object.getId();
				//currentLocation = dojo.doc.location.href.substr(0, dojo.doc.location.href.indexOf("#"));
	
				//TOGGLE PAGE_ORIENTED DESIGN
				if (!pageLoadAssembly) {
					launchNextGenPage("workAreaDijit=ibm/usmi/widgets/WorkArea&sourceToken=" + oid + "&selector=details");
				} else {
					launchNextGenPage("workAreaDijit=ibm/usmi/widgets/WorkArea&selector=details#sourceToken=" + oid);
				}
				*/
				
				if(objectContext.cmProvider){
					objectContext.cmProvider.getMenuCommand("nwmi_PropertiesNotebookDialog", objectContext, null);
				}
				
			}

		},
		

		executePropertyAction : function (/*Object*/objectContext, /*Property*/property) {
			// summary:
			//		Method which checks context for default server task and executes if needed
			//		else executes the callback method
			// description:
			//		Coming soon
			// objectContext:
			//		Coming soon
			if (this._daLogger.isFinest()) {
				this._daLogger.entering("", "executePropertyAction");
			}
			var defaultTask = property.getTask();
			if (defaultTask) {

				if (defaultTask.getType() === "javascript") {
					if (this._daLogger.isFinest()) {
						this._daLogger.finest("default property task " + defaultTask.getId(), "executePropertyAction");
					}

					var jsonLaunchParms = defaultTask.getLaunchJavascript();
					var launchParms = dojo.fromJson(jsonLaunchParms);
					launchParms.selection = objectContext.selection;

					if (launchParms.topic) {
						topic.publish(launchParms.topic, launchParms, defaultTask, objectContext);
					} else {
						if (this._daLogger.isWarn()) {
							this._daLogger.warn("default property task " + defaultTask.getId()
									+ "did not specify topic in launch parms", "executePropertyAction");
						}
					}
					return;
				} else if (defaultTask.isUrlLaunch()) {
					var url = defaultTask.getLaunchUrl();
					if (url) {
						var launchParms = {
							urlToLaunch : url,
							launchType : "URL"
						};
						topic.publish("DefaultURLLaunch", launchParms, defaultTask, objectContext);
					}
				}
			}
			if (this._daLogger.isFinest()) {
				this._daLogger.exiting("", "executePropertyAction");
			}
		},

		getDataSet : function () {
			return this._args._dataStore.dataset;
		},

		getSelection : function () {
			var selectedThings = this._args.selection, //        this._args.context.selection;
			currentSelection = null;
			if (selectedThings) {
				currentSelection = new Selection({
					version : this.getDataSet().getVersion()
				});
				currentSelection.addItems(selectedThings.getItemIds());
				currentSelection.addLinks(selectedThings.getLinkIds());
				currentSelection.addRelationships(selectedThings.getRelationshipIds());
			}
			return currentSelection;
		}
	});
});