/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

define("ibm/usmi/widgets/TaskPane", [
    "dojo/_base/declare",
    "dojo/_base/array",
    "dojo/_base/lang",
    "dojo/topic",
    "dijit/_WidgetBase",
    "dijit/_Container",
    "dojo/topic",
    "dojo/hash",
    // NextGen imports
    "ibm/usmi/widgets/TaskLink",
    "ibm/usmi/widgets/TaskCategory",
    "ibm/usmi/util/BaseWidget",
    "ibm/usmi/widgets/util/Constants"
], function(declare, array, lang, topic, _WidgetBase, _Container, topic, hash, TaskLink, TaskCategory, BaseWidget,
        Constants) {
    // module:
    //      ibm/usmi/widget/TaskPane
    // summary:
    //      The TaskPane is the container widget that manages the TaskCategories and Tasklinks for the Details Selector 
    //      The details selector template hierarchy  TaskPane->TaskCategory->TaskLink.
    //      He is a hash listener that handles bookmarks, direct url launches, visually selects / deselects task links, 
    //      and updates workarea titles.

    return declare([
        _WidgetBase,
        _Container,
        BaseWidget
    ], {
        // array holding all categories that have been added to taskPane
        _categories: [],

        // array holding all taskLinks that have been added to the categories 
        _links: [],

        // handle for subscribe, so it can be cleaned up on destroy
        subscription: null,

        // propagate tabIndex to TaskCategorys from declaration or invocation, default to zero
        tabIndex: "0",

        startup: function() {
            this.inherited(arguments);
            this._categories = this.getChildren();
            this._links = [];
            var _this = this;
            array.forEach(this._categories, function(category) {
                _this._links = _this._links.concat(category.getChildren());
            });

            this.subscription = this.subscribe(Constants.TOPIC_HASH_CHANGE, this._hashChanged);
        },

        refresh: function() {
            this._categories = this.getChildren();//this.getChildren();
            this._links = [];
            var _this = this;
            array.forEach(this._categories, function(category) {
                _this._links = _this._links.concat(category.getChildren());
            });
        },

        addCategory: function(category) {
            var g;
            if (category instanceof TaskCategory) {
                g = lang.mixin(category, {
                    tabIndex: this.tabIndex
                });
            } else {
                g = new TaskCategory(lang.mixin(category, {
                    tabIndex: this.tabIndex
                }));
            }

            this.addChild(g);
        },

        setLink: function(/*ibm.usmi.widgets.TaskLink*/link) {
            if (link.getParent().get("visible")) {
                array.forEach(this._links, function(item) {
                    item.select(false);
                });
                link.select(true);
            }
        },

        _hashChanged: function(/*hash*/hash) {
            array.forEach(this._links, function(link) {
                // if criteria exists, trying to render a related resource view, compare task id versus
                // hash releationship
                if (link.criteriaString) {
                    if (this.getUrlHashParm(Constants.HASH_TASK_RELATIONSHIP) === link.taskId) {
                        this._linkClicked(null, link);
                    }
                }
                // if not no criteria string, compare task id versus hash task selection parm
                else {
                    if (this.getUrlHashParm(Constants.HASH_TASK_SELECTION) === link.taskId) {
                        this._linkClicked(null, link);
                    }
                }
            }, this);
        },

        _linkClicked: function(event, link) {
            this.setLink(link);

            //publish the label, so workarea can render title
            topic.publish(Constants.TOPIC_TASK_TITLE_CHANGE, {
                resource: link.get("resourceLabel"),
                view: link.get("label"),
                loading: false,
                titleMaster: true
            });

            // if this wasn't kicked off by a link selection, must have been a bookmark or direct call,
            // need to kick off retarget event
            if (!link.get("wasClicked")) {
                link.fakeClick();
            }

            // reset variable
            link.resetWasClicked();
        },

        getCategory: function(/*String*/category) {
            var subset = array.filter(this.getChildren(), function(/*Widget*/child) {
                return child.isInstanceOf(TaskCategory) && child.category === category;
            }, this);
            return (subset && subset[0]) || null;
        },
        destroy: function() {
            if (this.subscription) {
                this.subscription.remove();
                this.subscription = null;
            }
            this.inherited(arguments);
        }
    });
});
