/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
/**
 * Finder.js is the main widget that shows up in the top right corner of NGui.
 * It takes over the idx search widget by having hooks in main.js and our idx
 * override of Header.js
 */

//There were complications of defining the finder as a templated widget due to being contained within the idx Header.js . 
define("ibm/usmi/widgets/finder/Finder", [
	"dojo/_base/lang",
	"dojo/dom",
	"dojo/json",
	"dojo/_base/array",
	"dojo/on",
	"dojo/_base/connect",
	"dojo/dom-construct",
	"dijit/registry",
	"dojo/_base/declare",
	"dijit/_WidgetBase",
	"dojo/topic",
	"dojo/cookie",
	"./FinderSearchDialog",
	"dijit/popup",
	"./FinderTabbedContainer",
	"dojo/i18n!ibm/usmi/nls/frameworkResources",
	"ibm/usmi/util/DataStore",
	"ibm/usmi/util/Utils",
	"./TaskDataStore",
	"./ResourceDataStore",
	"./GroupDataStore",
	"./FinderContextSearch"
], function (lang, dom, JSON, array, on, connect, domConstruct, registry, declare, _WidgetBase, topic, cookie,
		TooltipDialog, popup, TabContainer, MRI, DataStore, Utils, TaskDataStore, ResourceDataStore, GroupDataStore,
		FinderContextSearch) {
	return declare([ _WidgetBase
	], {
		// dialog
		tooltipShown : null,
		_searchDetailDlg : null, // finder results tooltip
		_tabbedContainer : null,
		// finder state variables
		_searchDlgIsOpen : false,
		_searchDlgInitialized : false,
		_latestSearchEntry : null,
		_currentView : '0',
		_searchTextHintString : null,
		//This is used if there is 1 search result only for current search, the item is put here so that if the user presses ENTER, the item will be launched automatically
		_currentResultCountArray : null,
		//used to get the finder collection, which contains non-targetted tasks, resources, and groups
		_dataStore : null,
		//connect handles in reference so we can clean them up later
		_dojoConnectHandles : null,
		//Refresh variables
		blockEvents : true,
		pendingEvents : false,
		initialized : false,
		retrievingData : false,
		searchStringEntered : null,
		searchOpen : false,
		origStyle : "dijitReset dijitInline dijitButtonNode",
		refreshStyle : "dijitReset dijitInline finderRefresh",
		lastRefresh : 0,
		manualRefresh : false,
		// Data stores containing cached lists
		_cachedDataStores : null,
		_cachedDataStoresIndexes : null,
		//indexed reference list of all finder items for quick retrieval
		_indexedItems : null,
		contextFinder : null,
		constructor : function () {
			this.inherited(arguments);
			this._cachedDataStores = [];
			this._cachedDataStoresIndexes = [
				'tasks',
				'resources',
				'groups',
				'contexttasks'
			];
			this._indexedItems = [];
			Utils.CMTraceTimeInit("finderTimer");
			this._searchTextHintString = MRI.FinderInitialText;
			this._currentSingleResultArray = {};
			this._dojoConnectHandles = [];
		},
		postCreate : function () {
			this.inherited(arguments);
			this.initializeFinderHtml();
			this._dataStore = new DataStore({
				datasetId : 'FinderCollection',
				initReadyCallback : lang.hitch(this, this.initializeDataStores)
			});
			this.contextFinder = new FinderContextSearch(this);
			this._dojoConnectHandles.push(topic.subscribe("CurrentResourceSelection", lang.hitch(this.contextFinder,
					this.contextFinder.changeSelection)));
		},
		incrementRefreshMinute : function () {
			if (this.lastRefresh === 15) { //since registering for all changes is very expensive, we listen for a few key ones and refresh every 15 minutes to keep a fresh cache
				this._refreshCache();
			} else {
				this.lastRefresh = this.lastRefresh + 1;
				if (this._searchDetailDlg && this._searchDetailDlg.refreshImage) {
					this._searchDetailDlg.changeRefreshCount();
				}
			}
			setTimeout(lang.hitch(this, 'incrementRefreshMinute'), 60000);
		},
		initializeDataStores : function () {
			this.initializeResourcesAndGroups(true);
			setTimeout(lang.hitch(this, 'incrementRefreshMinute'), 60000);
		},
		//If finder cache is refreshing, disable the finder and change styles to reflect this
		_doRefreshProcessing : function () {
			if (this._searchDlgIsOpen) {
				this.searchOpen = true;
			}
			dom.byId("widget_findersearchBox").nextSibling.childNodes[0].className = this.refreshStyle;
			if (this._latestSearchEntry) {
				cookie("finder.lastSearch", this._latestSearchEntry, {
					expires : 1
				});
			}
			var inputField = registry.byId("findersearchBox").domNode.childNodes[0].childNodes[0];
			this.closeSearchDetailDlg();
			inputField.value = MRI.FinderRefreshing;
			inputField.disabled = true;
		},
		//User clicked refresh, process manual refresh of finder cache
		_manualRefresh : function () {
			if (this._searchDlgIsOpen) {
				this.searchOpen = true;
			}
			this.manualRefresh = true;
			this._doRefreshProcessing();
			this._refreshCache();
		},
		//event driven or manually driven refresh request, clean up old cache and request new data from server
		_refreshCache : function () {
			if (this.blockEvents) { //Refresh is in progress, don't refresh
				this.pendingEvents = true; //save the need to refresh again later
				return;
			}
			this.lastRefresh = 0;
			this.blockEvents = true;
			this.pendingEvents = false;
			if (!this.manualRefresh) {
				this._doRefreshProcessing();
			} else {
				this.manualRefresh = false;
			}
			this._dataStore.getDataset().removeEventListener(this);
			this._dataStore.destroy();
			if (this._cachedDataStores.resources) {
				this._cachedDataStores.resources.destroy();
				this._cachedDataStores.resources = null;
			}
			if (this._cachedDataStores.groups) {
				this._cachedDataStores.groups.destroy();
				this._cachedDataStores.groups = null;
			}
			this._dataStore = new DataStore({
				datasetId : 'FinderCollection',
				initReadyCallback : lang.hitch(this, this.initializeResourcesAndGroups)
			});
		},
		initializeResourcesAndGroups : function (initial) {
			this._dataStore.getDataset().addEventListener(this, lang.hitch(this, this._refreshCache));
			this._cachedDataStores.tasks = new TaskDataStore('tasks', this.retrieveDatasetCache([
				'tasks',
				'jobs',
				'ngenTask',
				'dialog',
				'urlTasks'
			]));
			this._cachedDataStores.resources = new ResourceDataStore('resources', this
					.retrieveDatasetCache('resources'));
			this._cachedDataStores.groups = new GroupDataStore('groups', this.retrieveDatasetCache('groups'));
			window.setTimeout(lang.hitch(this, this._unblockEvents), '60000');
			if (initial !== true) {
				this.initializeFinderHtml();
				dom.byId("widget_findersearchBox").nextSibling.childNodes[0].className = this.origStyle;
				registry.byId("findersearchBox").domNode.childNodes[0].childNodes[0].disabled = false;
				if (this.searchOpen) {
					this.searchOpen = false;
					this.finderKeyPressed({});
				}
			}
		},
		_unblockEvents : function () {
			this.blockEvents = false; //Refresh is done, we can now listen for events again
			if (this.pendingEvents) { //did any more events come in during our refresh? if so, refresh again
				this._refreshCache();
			}
		},
		retrieveDatasetCache : function (
		/* String or array of strings*/types) {
			var items = [], idx;
			array.forEach(this._dataStore.getItems(), lang.hitch(this, function (entry, i) {
				var matches = false;
				var type = entry.getProperty('FINDER_TYPE_COLUMN').getValue();
				if (types instanceof Array) {
					for (idx = 0; idx < types.length; idx++) {
						if (types[idx] === type) {
							matches = true;
						}
					}
				} else {
					matches = type === types;
				}
				if (matches) {
					var iconString = null;
					if (entry.getProperty('FINDER_ICON_COLUMN')) {
						iconString = entry.getProperty('FINDER_ICON_COLUMN').getValue();
					}
					var itId = entry.getProperty('FINDER_ID_COLUMN').getValue();
					items.push({
						id : itId,
						label : entry.getProperty('FINDER_NAME_COLUMN').getValue(),
						category : entry.getProperty('FINDER_CATEGORY_COLUMN').getValue(),
						icon : iconString
					});
					this._indexedItems[itId] = entry;
				}
			}));
			return items;
		},
		initializeFinderHtml : function () {
			var inputFieldWidget = registry.byId("findersearchBox");
			var inputField = inputFieldWidget.domNode.childNodes[0].childNodes[0];
			var lastEnteredSearch = cookie("finder.lastSearch");
			if (lastEnteredSearch) {
				inputField.value = lastEnteredSearch;
				cookie("finder.lastSearch", null, {
					expires : -1
				});
			} else {
				inputField.value = this._searchTextHintString;
			}
			if (!this.initialized) {
				this._dojoConnectHandles.push(on(inputField, "blur", lang.hitch(this, this.doSearchBoxBlur)));
				this._dojoConnectHandles.push(on(inputField, "focus", lang.hitch(this, this.doSearchBoxFocus)));
				this._dojoConnectHandles.push(on(inputField, "keyup", lang.hitch(this, this.finderKeyPressed)));
				this.initialized = true;
			}
			if (registry.byId("finderdialog")) {
				registry.byId("finderdialog").destroy();
			}
			this.createTooltipDialog();
		},
		createTooltipDialog : function () {
			this._searchDetailDlg = new TooltipDialog({
				id : "finderdialog",
				'class' : "finderdialog",
				finderWidget : this,
				resize : function () {}/* HACK, figure out why resize kills the dialog*/
			});
			this._searchDetailDlg.startup();
		},
		fetchFailed : function (error, request) {
			Utils.CMFinest('finder: ' + error);
		},
		doSearchBoxBlur : function () {
			if (registry.byId("findersearchBox").get('value') === "") {
				registry.byId("findersearchBox").set('value', this._searchTextHintString);
			}
		},
		doSearchBoxFocus : function () {
			if (registry.byId("findersearchBox").get('value') === this._searchTextHintString) {
				registry.byId("findersearchBox").set('value', "");
			}
			registry.byId("findersearchBox").focus();
		},
		closeSearchDetailDlg : function (keepFocusOnInput) {
			if (this._searchDetailDlg) {
				popup.close(this._searchDetailDlg);
				this._searchDetailDlg.destroyRecursive();
				this.createTooltipDialog();
			}
			this._searchDlgIsOpen = false;
			this._latestSearchEntry = '';
			var inputFieldWidget = registry.byId("findersearchBox");
			var inputField = inputFieldWidget.domNode.childNodes[0].childNodes[0];
			if (keepFocusOnInput !== true) {
				inputField.value = this._searchTextHintString;
				inputField.blur();
			}
			array.forEach(this._cachedDataStoresIndexes, lang.hitch(this, function (cachedDataStoreIndex) {
				if (this._cachedDataStores[cachedDataStoreIndex]) {
					this._cachedDataStores[cachedDataStoreIndex].clearTooltips();
				}
			}));
		},
		doSearch : function (dataStore, moreItemsTask, moreItemsText, typeOfSearch, moreItemsNotClickable, myitems) {
			if (myitems.length === 1) {
				this._currentSingleResultArray[typeOfSearch] = myitems[0].id;
			} else {
				this._currentSingleResultArray[typeOfSearch] = null;
			}
			Utils.CMTraceTimeDelta("finderTimer", "doSearch" + typeOfSearch, true);
			var searchString = registry.byId("findersearchBox").get('value');
			if (searchString === "" || searchString === this._searchTextHintString) {
				return;
			}
			if (myitems) {
				if (myitems.length > 0) {
					dataStore.clearSelection();
					Utils.CMTraceTimeDelta("finderTimer", "processing results", true);
					dataStore.populateCategoriesWithSelectedItems(myitems);
					Utils.CMTraceTimeDelta("finderTimer", "results populated", true);
					this.initializeDialogConstruct(typeOfSearch, myitems.length);
					Utils.CMTraceTimeDelta("finderTimer", "initialized dialog", true);
					dataStore.createCategoryHTMLElements(myitems.length, this, moreItemsTask, moreItemsText,
							moreItemsNotClickable);
					Utils.CMTraceTimeDelta("finderTimer", "results UI created", true);
				} else {
					this.initializeDialogConstruct(typeOfSearch);
					var noMatchesFoundText;
					if (typeOfSearch === 'resources') {
						noMatchesFoundText = MRI.NoResourcesFound;
					} else if (typeOfSearch === 'tasks' || typeOfSearch === 'contexttasks') {
						noMatchesFoundText = MRI.NoTasksFound;
					} else if (typeOfSearch === 'groups') {
						noMatchesFoundText = MRI.NoGroupsFound;
					}
					domConstruct.create("div", {
						innerHTML : noMatchesFoundText
					}, dom.byId('finder' + typeOfSearch + "_containerDiv"));
				}
			}
			if (typeOfSearch === 'resources') {
				this._tabbedContainer.init();
			}
		},
		doClick : function (event, id) {
			var selectedTaskId, clickType = 1, url, launchParms;
			if (!event) {
				selectedTaskId = id;
			} else {
				clickType = event.which;
				selectedTaskId = event.currentTarget.id;
			}
			this.closeSearchDetailDlg();
			Utils.CMFinest('finder: doClick' + selectedTaskId);
			var type = selectedTaskId.split('#_')[0];
			var itemId = selectedTaskId.split('#_')[1];
			if (type.indexOf('categorySection') !== -1) {
				var category = this._cachedDataStores.resources.getCategories()[itemId];
				var ids = category.getIds();
				selectedTaskId = 'resourceList#_';
				var i;
				for (i = 0; i < ids.length; i++) {
					if (i !== 0) {
						selectedTaskId = selectedTaskId + ",";
					}
					selectedTaskId = selectedTaskId + ids[i];
				}
			}
			if (itemId.indexOf("com.ibm.usmi.console.taskmap") !== -1) {
				launchNextGenPage("workAreaDijit=ibm/usmi/widgets/ResourceNavigator&dataset=GROUP:USMI_TASK_MAP");
				return;
			} else if (itemId.indexOf("com.ibm.usmi.console.tasks.QuickFindTask") !== -1) {
				launchNextGenPage("workAreaDijit=ibm/usmi/widgets/ResourceNavigator&dataset=GROUP:ALL_ENDPOINTS");
				return;
			}
			// get item from dataset, since our cache doesn't have all the info
			var item = this._dataStore.getItemFromProperty("FINDER_ID_COLUMN", itemId);
			if (type === 'tasks') {
				if (item) {
					var taskType = item.getProperty('FINDER_TYPE_COLUMN').getValue();
					var js = "";
					if (item.getProperty("FINDER_LAUNCH_JAVASCRIPT_COLUMN")) {
						js = "{" + item.getProperty("FINDER_LAUNCH_JAVASCRIPT_COLUMN").getValue() + "}";
					}
					if (taskType === 'jobs') {
						// non interactive task, launch launcher dialog widget
						launchParms = {
							modulePath : "ibm.usmi.scheduler",
							dijit : "JobLaunchWizard",
							topic : "DefaultNoninteractiveTask",
							taskID : itemId,
							dialogTitle : item.getProperty('FINDER_NAME_COLUMN').getValue()
						};
						topic.publish("DefaultDialogLaunch", launchParms);
					} else {
						// for any launchable task, we have launch javascript from server side task, publish a topic to launch it
						launchParms = null;
						try {
							launchParms = JSON.parse(js);
						} catch (e) {
							console.error(itemId + "task has incorrect javascript defined in its"
									+ " getLaunchParameters/getLaunchJavascript method");
							launchParms = dojo.fromJson(js);
						}
						//send empty context, null seems to blow up in URL launch handler
						topic.publish(launchParms.topic, launchParms, null, {});
					}
				}
			} else if (type === 'resources') {
				//var resourceTypeProperty = item.getProperty('FINDER_RESOURCE_TYPE_COLUMN');
				//var chassisMapProperty = item.getProperty('FINDER_RESOURCE_TYPE_CHASSISMAP_COLUMN');
				//by default, launch into details view
				url = "workAreaDijit=ibm/usmi/widgets/WorkArea&sourceToken=" + itemId + "&selector=details";
				// We can't link to the chassis map due to a bug in director - link to details for now
				/*
				if (resourceTypeProperty) {
					var resourceType = (resourceTypeProperty.getValue());
					if (resourceType === 'System Chassis'
							|| (chassisMapProperty && chassisMapProperty.getValue() === 'true')) {
						//we detected we have a resource contained in the chassis map, launch there instead
						url = "workAreaDijit=ibm/usmi/widgets/WorkArea&sourceToken=" + itemId
								+ "&selector=ibm/usmi/dashboard/ResourceDashBoard&selectedGroup=chassis";
					}
				}*/
				if (clickType > 1) {
					window.open(window.location.href.split("#")[0] + "#" + url);
				} else {
					launchNextGenPage(url);
				}
			} else if (type === 'groups') {
				url = "workAreaDijit=ibm/usmi/widgets/ResourceNavigator&dataset=GROUP:" + itemId;
				if (clickType > 1) {
					window.open(window.location.href.split("#")[0] + "#" + url);
				} else {
					launchNextGenPage(url);
				}
			} else if (type === 'contexttasks') {
				this.contextFinder.executeTask(this._indexedItems[itemId + "#_context"]);
			}
		},
		setTooltip : function (id) {
			this.tooltipShown = id;
		},
		finderKeyPressed : function (event) {
			Utils.CMFinest('finder: keyPressed');
			Utils.CMTraceTimeDelta("finderTimer2", "keyPressed", true);
			if (this.tooltipShown) {
				this.tooltipShown.close();
			}
			if (event.keyCode === 27) {
				registry.byId("findersearchBox").set("value", "");
			} else if (event.keyCode === 13) {
				var resultId = this._currentSingleResultArray[this._currentView];
				if (resultId) {
					this.doClick(null, this._currentView + "#_" + resultId);
				}
				return;
			}
			this.searchStringEntered = registry.byId("findersearchBox").get('value');
			if (this._latestSearchEntry === this.searchStringEntered) {
				return;
			} else {
				this._latestSearchEntry = this.searchStringEntered;
			}
			if ((this.searchStringEntered === "") || (this.searchStringEntered === this._searchTextHintString)) {
				this.closeSearchDetailDlg(true);
				return;
			}
			if (!this._searchDlgIsOpen || this._searchDetailDlg._isShown() === false) {
				popup.open({
					popup : this._searchDetailDlg,
					around : registry.byId("findersearchBox").domNode,
					onCancel : lang.hitch(this, this.closeSearchDetailDlg),
					parent : registry.byId("findersearchBox")
				});
			}
			this._searchDlgIsOpen = true;
			registry.byId("findersearchBox").focus();
			Utils.CMFinest('finder: query: ' + this.searchStringEntered);
			this._searchDlgInitialized = false;
			this.contextFinder.determineContextSearch();
			if (this.retrievingData === true) {
				return;
			}
			this.initiateSearch(0);
		},
		initiateSearch : function (state/* ,keyPressedNumber*/) {
			var contextInitializedOrInactive = false, datastoresReady = false;
			if (this.contextFinder.selectedOids) {
				if (this._cachedDataStores.contexttasks) {
					contextInitializedOrInactive = true;
				}
			} else {
				contextInitializedOrInactive = true;
			}
			datastoresReady = this._cachedDataStores.tasks && this._cachedDataStores.groups
					&& this._cachedDataStores.resources;
			if (datastoresReady && contextInitializedOrInactive) {
				if (dom.byId('searchingText')) {
					domConstruct.destroy('searchingText');
					this.retrievingData = false;
				}
				this._cachedDataStores.tasks.initiateTaskSearch(this.searchStringEntered, this);
				this._cachedDataStores.groups.initiateTaskSearch(this.searchStringEntered, this);
				this._cachedDataStores.resources.initiateTaskSearch(this.searchStringEntered, this);
				if (this._cachedDataStores.contexttasks && this.contextFinder.selectedOids
						&& this.contextFinder.selectedOids !== "") {
					if (dom.byId("findercontexttasks_containerDiv")) {
						dom.byId("findercontexttasks_containerDiv").style.display = "block";
						dom.byId("findercontexttasks_border").style.display = "block";
						dom.byId("findertasksTitle").style.display = "block";
					}
					this._cachedDataStores.contexttasks.initiateTaskSearch(this.searchStringEntered, this);
				} else {
					if (dom.byId("findercontexttasks_containerDiv")) {
						dom.byId("findercontexttasks_containerDiv").style.display = "none";
						dom.byId("findercontexttasks_border").style.display = "none";
						dom.byId("findertasksTitle").style.display = "none";
					}
				}
			} else {
				this.retrievingData = true;
				var stateNumber = state % 3;
				if (!dom.byId('searchingText')) {
					this._searchDetailDlg.searching(stateNumber);
				} else {
					dom.byId('searchingText').innerHTML = MRI['Searching' + stateNumber];
				}
				setTimeout(lang.hitch(this, 'initiateSearch', state + 1 /* , keyPressedNumber*/), 200);
			}
		},
		initializeDialogConstruct : function (typeOfSearch, items) {
			if (this._searchDlgInitialized === false) {
				this._searchDlgInitialized = true;
				this._searchDetailDlg.initialize();
				if (registry.byId("findertab")) {
					registry.byId("findertab").destroyRecursive();
				}
				var mainDivTabContainer = domConstruct.create("div", {
					id : 'mainDivTabContainer'
				}, this._searchDetailDlg.containerNode);
				this._tabbedContainer = new TabContainer({
					id : "findertab",
					style : "width:100%;",
					doLayout : true,
					useMenu : false,
					useSlider : false,
					finderObject : this,
					contextFinder : this.contextFinder
				}, mainDivTabContainer);
				var lastSelectedTab = cookie("finder.currentView");
				if (lastSelectedTab && lastSelectedTab.length > 0) {
					this._currentView = lastSelectedTab;
				}
				this._tabbedContainer.addChildrenTabs(this._currentView);
				this._dojoConnectHandles.push(connect.connect(this._tabbedContainer, "selectChild", lang.hitch(this,
						this.tabSelected)));
			}
			// add the count in parenthesis to the tabs
			var numItems;
			if (items) {
				numItems = items;
			} else {
				numItems = 0;
			}
			if (this._tabbedContainer) {
				this._tabbedContainer.setTabbedTitleCounts(typeOfSearch, numItems);
			}
		},
		tabSelected : function (child) {
			if (child.content.id.indexOf('tasks') !== -1) {
				this._currentView = 'tasks';
			} else if (child.content.id.indexOf('resources') !== -1) {
				this._currentView = 'resources';
			} else if (child.content.id.indexOf('groups') !== -1) {
				this._currentView = 'groups';
			}
			cookie("finder.currentView", this._currentView, {
				expires : 30
			});
			Utils.CMFinest('finder: ' + child);
			Utils.CMFinest('finder: ' + this._currentView);
		},
		destroyRecursive : function () {
			this.destroy();
		},
		destroy : function () {
			this.inherited(arguments);
			if (this._searchDetailDlg) {
				popup.close(this._searchDetailDlg);
				this._searchDetailDlg.destroyRecursive();
				this._searchDetailDlg = null;
			}
			if (this._tabbedContainer) {
				this._tabbedContainer.destroyRecursive();
				this._tabbedContainer = null;
			}
			if (this._dataStore) {
				this._dataStore.getDataset().removeEventListener(this);
				this._dataStore.destroy();
				this._dataStore = null;
			}
			if (this.contextFinder) {
				this.contextFinder.destroy();
			}
			array.forEach(this._cachedDataStoresIndexes, lang.hitch(this, function (cachedDataStoreIndex) {
				if (this._cachedDataStores[cachedDataStoreIndex]) {
					this._cachedDataStores[cachedDataStoreIndex].destroy();
					this._cachedDataStores[cachedDataStoreIndex] = null;
				}
			}));
			this._cachedDataStores = null;
			this._cachedDataStoresIndexes = null;
			if (this._dojoConnectHandles) {
				array.forEach(this._dojoConnectHandles, function (handle) {
					handle.remove();
				});
				this._dojoConnectHandles = null;
			}
		}
	});
});
