/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/widgets/util/DefaultContextMenuProvider", [
    "dojo/_base/declare",
    // TWL imports
    "ibm/tivoli/tip/navwidget/dijit/RESTContextMenuProvider"
],
        function(declare, RESTContextMenuProvider) {
            // module:
            //      ibm/usmi/widget/util/DefaultContextMenuProvider
            // summary:
            //      DefaultContextMenuProvider extends RESTContextMenuProvider which knows how to use the REST Service 
            //      to ask the data provider for a list of tasks for the selected resources.  It then takes these tasks 
            //      and uses the information from them to create the JSON format to populate the dijit.Menu managed by AsynchContextMenu.  
            //      It also knows how to execute tasks clientside as well as ones to be sent back through the REST Service to the data provider.
            //
            //      We use it by filtering out the the tasks, task types that were returned from from server that we dont want.
            return declare([ RESTContextMenuProvider
            ],
                    {
                        _handle: null,

                        filterNavigationTasks: function(/*ibm.tivoli.navmodel.Task[]*/tasks) {
                            // summary:
                            //    Return a filtered collection of tasks
                            //    Filter out related resources (parent has id RELATED_RESOURCES,
                            //       children have id RELATIONSHIP:*
                            //    Filter out related resources (parent has id PERSPECTIVES,
                            //       children have id PERSPECTIVE:*
                            //    Note:  Separators have no task id.
                            //
                            var filteredTasks = [], i;
                            for (i = 0; i < tasks.length; i++) {
                                if ((tasks[i].getId() === undefined)
                                        || ((tasks[i].getId() !== "RELATED_RESOURCES")
                                                && (tasks[i].getId().indexOf("RELATIONSHIP:") !== 0)
                                                && (tasks[i].getId() !== "PERSPECTIVES")
                                                && (tasks[i].getId().indexOf("PERSPECTIVE:") !== 0) && (tasks[i]
                                                .getId() !== "VIEW_MEMBERS"))) {
                                    filteredTasks.push(tasks[i]);
                                }
                            }
                            return filteredTasks;
                        }
                    });
        });