/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

define("ibm/usmi/widgets/util/PropertyRenderer", [
	"dojo/_base/declare",
	"dojo/dom-attr",
	"dojo/_base/window",
	"dojo/dom-construct",
	"dojo/dom-class",
	"dijit/ProgressBar",
	//TWL imports
	"ibm/tivoli/navmodel/Property",
	"ibm/tivoli/navmodel/Status",
	"ibm/tivoli/navmodel/ItemStyle",
	"ibm/tivoli/navmodel/Item",
	"ibm/tivoli/navmodel/Link",
	"ibm/tivoli/navmodel/Relationship",
	"ibm/tivoli/tip/navwidget/dojo/NavWidgetUtil",
	"ibm/tivoli/tip/navwidget/properties/PropertyRenderer",
	// Next Gen
	"ibm/usmi/widgets/util/Constants"
],

function (declare, attr, window, construct, domClass, ProgressBar, Property, Status, ItemStyle, Item, Link,
		Relationship, NavWidgetUtil, PropertyRenderer, Constants) {
	// module:
	//      ibm/usmi/widget/util/PropertyRenderer
	// summary:
	//      The Details Selector is a common selector provided by the framework for displaying the properties and 
	//      related resources of an object.  This should be used in most cases where you're inclined to use a PropertiesNotebook, 
	//      as it provides a common interface and navigation for resource details.

	return declare([ PropertyRenderer
	], {

		//***************************
		// "Public Methods"
		//***************************

		constructor: function () {
			//Call superclass
			this.inherited(arguments);
		},

		getRenderedPropertyNode: function (/*ibm.tivoli.navmodel.Item*/object, prop, plugviewref, rowID, cell) {
			// summary:
			//        This is editting the TWL version so we don't output the second image on the title in the 
			//		  detail selector portion first check if there is a valid value
			
			if (prop.getValueState()==="ok" && prop.getValueType() === "itemName") {
				var contentNode = window.doc.createElement("div"), threshold, altText;

				//a threshold callback is defined, so use it to check if we need
				//to threshold this value and ADD an image which we normally wouldnt render
				if (this.propThresholdCallback) {
					threshold = this.propThresholdCallback(prop, object);

					//only if an image is defined on the threshold for this value
					if (threshold && threshold.icon) {
						altText = (threshold.label ? threshold.label.value : null);
						if (!altText) {
							altText = prop.getDisplayValue();
						}

						//render the node, and add it
						construct.place(this._getImageContentNode(plugviewref, prop, object, "", "16px", "16px"),
								contentNode, "last", altText);
					}
				}

				if (prop.getValueUnits()) {
					construct.place(this._getLabelContentNode(prop.getDisplayValue() + " " + prop.getValueUnits(),
							prop, object, plugviewref), contentNode, "last");
				} else {
					construct.place(this._getLabelContentNode(prop.getDisplayValue(), prop, object, plugviewref),
							contentNode, "last");
				}

				//color background if thresholding needed
				this._thresholdBackground(cell, prop, object);

				return contentNode;
			}

			return this.inherited(arguments);
		}
	});
});
