(function(){

var factory = function(dojo_declare, dojo_lang, dojo_array, dojo_window, dojo_html, dojo_event, dojo_connect, dojo_keys, dijit_registry, dijit_Templated, dijit_layout_LayoutWidget, dijit_layout_ContentPane, idx_html, idx_util, idx_layout_BorderContainer, idx_layout_MoveableTabContainer, idx_layout_TitlePane, idx_layout_Dockable, idx_layout_DockAreaMixin){
/**
 * @name idx.layout.DockTabContainer
 * @class Tabbed dock area based on TabContainer
 * @augments idx.layout.MoveableTabContainer
 * @augments idx.layout._DockAreaMixin
 */
var DockTabContainer = dojo_declare("idx.layout.DockTabContainer", [idx_layout_MoveableTabContainer, idx_layout_DockAreaMixin],
/**@lends idx.layout.DockTabContainer#*/
{

	/**
	 * Tab layout is always horizontal so set collision check dimension to x
	 */
	postMixInProperties: function() {
		this.inherited(arguments);
		this._dim = "x";	// tabs are always horizontal
	},
	
	/**
	 * Sets up CSS class and event handler.
	 */
	postCreate: function() {
		this.inherited(arguments);
		dojo_html.addClass(this.domNode, "idxDockTabContainer");
		this.connect(this.tablist, "_onMove", "_onTabMove");
	},

	/**
	 * Computes coordinates of each of my children, relative to my own position.
	 * This will later be used when calculating insert index.
	 * @private
	 */
	_computeBounds: function() {
		var rootPos = dojo_html.position(this.tablist.domNode);
		this._bounds = [];
		dojo_array.forEach(this.tablist.getChildren(), dojo_lang.hitch(this, function(child) {
			var pos = dojo_html.position(child.domNode);
			this._bounds.push({x: pos.x - rootPos.x + pos.w / 2, y: pos.y - rootPos.y + pos.h / 2});
		}));
	},

	/**
	 * Detects dragging out to undock.
	 * @param {Object} msg
	 * @private
	 */
	_onTabMove: function(msg) {
		this.inherited(arguments);
		// check to see if the user dragged the tab out of myself
		if (this.tablist._dragOut) {
			this.tablist._movingTab = false;
			this._undock(msg.content, msg.event);
		}
	},

	/**
	 * Undocks a child pane.
	 * @param {Object} pane
	 * @param {Object} evt
	 * @private
	 */
	_undock: function(pane, evt) {
		this.removeChild(pane);
		this.domNode.parentNode.appendChild(pane.domNode);
		dojo_html.removeClass(pane.domNode, "dijitTabPane");
		pane.set("selected", false);
		pane._onDragMouseDown(evt);
		pane._offsetX = 30;
		pane._offsetY = 10;
		pane.position(evt.clientX, evt.clientY);
		pane._startMove(evt);
	}	
});

return DockTabContainer;

}; // end of factory

var version = (window["dojo"] && dojo.version);
if(version && version.major == 1 && version.minor == 6){
	dojo.provide("idx.layout.DockTabContainer");
	dojo.require("dijit._Templated");
	dojo.require("dijit.layout._LayoutWidget");
	dojo.require("dijit.layout.ContentPane");
	dojo.require("idx.html");
	dojo.require("idx.util");
	dojo.require("idx.layout.BorderContainer");
	dojo.require("idx.layout.MoveableTabContainer");
	dojo.require("idx.layout.TitlePane");
	dojo.require("idx.layout._Dockable");
	dojo.require("idx.layout._DockAreaMixin");
	var dojo_event = {stop: dojo.stopEvent};
	factory(dojo.declare, dojo, dojo, dojo, dojo, dojo_event, dojo, dojo.keys, dijit, dijit._Templated, dijit.layout._LayoutWidget, dijit.layout.ContentPane, idx, idx.util, idx.layout.BorderContainer, idx.layout.MoveableTabContainer, idx.layout.TitlePane, idx.layout._Dockable, idx.layout._DockAreaMixin);
}else{
	define("idx/layout/DockTabContainer", [
		"dojo/_base/declare",
		"dojo/_base/lang",
		"dojo/_base/array",
		"dojo/_base/window",
		"dojo/_base/html",
		"dojo/_base/event",
		"dojo/_base/connect",
		"dojo/keys",
		"dijit/registry",
		"dijit/_Templated",
		"dijit/layout/_LayoutWidget",
		"dijit/layout/ContentPane",
		"idx/html",
		"idx/util",
		"idx/layout/BorderContainer",
		"idx/layout/MoveableTabContainer",
		"idx/layout/TitlePane",
		"idx/layout/_Dockable",
		"idx/layout/_DockAreaMixin"
	], factory);
}

})();
